#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_bgp_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_bgp_global
short_description: BGP Global resource module.
description:
- This module manages global BGP configuration on devices running Cisco NX-OS.
version_added: 1.4.0
notes:
- Tested against NX-OS 9.3.6.
- Unsupported for Cisco MDS
- This module works with connection C(network_cli) and C(httpapi).
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section '^router bgp').
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of BGP process configuration.
    type: dict
    suboptions:
      as_number:
        description: Autonomous System Number of the router.
        type: str
      affinity_group:
        description: Configure an affinity group.
        type: dict
        suboptions:
          group_id:
            description: Affinity Group ID.
            type: int
      bestpath: &bestpath
        description: Define the default bestpath selection algorithm.
        type: dict
        suboptions:
          always_compare_med:
            description: Compare MED on paths from different AS.
            type: bool
          as_path:
            description: AS-Path.
            type: dict
            suboptions:
              ignore:
                description: Ignore AS-Path during bestpath selection.
                type: bool
              multipath_relax:
                description: Relax AS-Path restriction when choosing multipaths.
                type: bool
          compare_neighborid:
            description: When more paths are available than max path config, use neighborid as tie-breaker.
            type: bool
          compare_routerid:
            description: Compare router-id for identical EBGP paths.
            type: bool
          cost_community_ignore:
            description: Ignore cost communities in bestpath selection.
            type: bool
          igp_metric_ignore:
            description: Ignore IGP metric for next-hop during bestpath selection.
            type: bool
          med:
            description: MED
            type: dict
            suboptions:
              confed:
                description: Compare MED only from paths originated from within a confederation.
                type: bool
              missing_as_worst:
                description: Treat missing MED as highest MED.
                type: bool
              non_deterministic:
                description: Not always pick the best-MED path among paths from same AS.
                type: bool
      cluster_id: &cluster_id
        description: Configure Route Reflector Cluster-ID.
        type: str
      confederation: &confederation
        description: AS confederation parameters.
        type: dict
        suboptions:
          identifier:
            description: Set routing domain confederation AS.
            type: str
          peers:
            description: Peer ASs in BGP confederation.
            type: list
            elements: str
      disable_policy_batching:
        description: Disable batching evaluation of outbound policy for a peer.
        type: dict
        suboptions:
          set:
            description: Set policy batching.
            type: bool
          ipv4:
            description: IPv4 address-family settings.
            type: dict
            suboptions:
              prefix_list:
                description: Name of prefix-list to apply.
                type: str
          ipv6:
            description: IPv6 address-family settings.
            type: dict
            suboptions:
              prefix_list:
                description: Name of prefix-list to apply.
                type: str
          nexthop:
            description: Batching based on nexthop.
            type: bool
      dynamic_med_interval:
        description: Sets the interval for dampening of med changes.
        type: int
      enforce_first_as:
        description: Enforce neighbor AS is the first AS in AS-PATH attribute (EBGP).
        type: bool
      enhanced_error:
        description: Enable BGP Enhanced error handling.
        type: bool
      fabric_soo:
        description: Fabric site of origin.
        type: str
      fast_external_fallover:
        description: Immediately reset the session if the link to a directly connected BGP peer goes down.
        type: bool
      flush_routes:
        description: Flush routes in RIB upon controlled restart.
        type: bool
      graceful_restart: &graceful_restart
        description: Configure Graceful Restart functionality.
        type: dict
        suboptions:
          set:
            description: Enable graceful-restart.
            type: bool
          restart_time:
            description: Maximum time for restart advertised to peers.
            type: int
          stalepath_time:
            description: Maximum time to keep a restarting peer's stale routes.
            type: int
          helper:
            description: Configure Graceful Restart Helper mode functionality.
            type: bool
      graceful_shutdown:
        description: Graceful-shutdown for BGP protocol.
        type: dict
        suboptions:
          activate:
            description: Send graceful-shutdown community on all routes.
            type: dict
            suboptions:
              set:
                description: Activate graceful-shutdown.
                type: bool
              route_map:
                description: Apply route-map to modify attributes for outbound.
                type: str
          aware:
            description: Lower preference of routes carrying graceful-shutdown community.
            type: bool
      isolate:
        description: Isolate this router from BGP perspective.
        type: dict
        suboptions:
          set:
            description: Withdraw remote BGP routes to isolate this router.
            type: bool
          include_local:
            description: Withdraw both local and remote BGP routes.
            type: bool
      log_neighbor_changes: &log_nbr
        description: Log a message for neighbor up/down event.
        type: bool
      maxas_limit: &maxas_limit
        description: Allow AS-PATH attribute from EBGP neighbor imposing a limit on number of ASes.
        type: int
      neighbors: &nbr
        description: Configure BGP neighbors.
        type: list
        elements: dict
        suboptions:
          neighbor_address:
            description: IP address/Prefix of the neighbor or interface.
            type: str
            required: true
          bfd:
            description: Bidirectional Fast Detection for the neighbor.
            type: dict
            suboptions:
              set:
                description: Set BFD for this neighbor.
                type: bool
              singlehop:
                description: Single-hop session.
                type: bool
              multihop:
                description: Multihop session.
                type: dict
                suboptions:
                  set:
                    description: Set BFD multihop.
                    type: bool
                  interval:
                    description: Configure BFD session interval parameters.
                    type: dict
                    suboptions:
                      tx_interval:
                        description: TX interval in milliseconds.
                        type: int
                      min_rx_interval:
                        description: Minimum RX interval.
                        type: int
                      multiplier:
                        description: Detect Multiplier.
                        type: int
          neighbor_affinity_group:
            description: Configure an affinity group.
            type: dict
            suboptions:
              group_id:
                description: Affinity Group ID.
                type: int
          bmp_activate_server:
            description: Specify server ID for activating BMP monitoring for the peer.
            type: int
          capability:
            description: Capability.
            type: dict
            suboptions:
              suppress_4_byte_as:
                description: Suppress 4-byte AS Capability.
                type: bool
          description:
            description: Neighbor specific description.
            type: str
          disable_connected_check:
            description: Disable check for directly connected peer.
            type: bool
          dont_capability_negotiate:
            description: Don't negotiate capability with this neighbor.
            type: bool
          dscp:
            description: Set dscp value for tcp transport.
            type: str
          dynamic_capability:
            description: Dynamic Capability
            type: bool
          ebgp_multihop:
            description: Specify multihop TTL for remote peer.
            type: int
          graceful_shutdown:
            description: Graceful-shutdown for this neighbor.
            type: dict
            suboptions:
              activate:
                description: Send graceful-shutdown community.
                type: dict
                suboptions:
                  set:
                    description: Set activate.
                    type: bool
                  route_map:
                    description: Apply route-map to modify attributes for outbound.
                    type: str
          inherit:
            description: Inherit a template.
            type: dict
            suboptions:
              peer:
                description: Peer template to inherit.
                type: str
              peer_session:
                description: Peer-session template to inherit.
                type: str
          local_as:
            description:
            - Specify the local-as number for the eBGP neighbor.
            - B(Deprecated), Use local_as_config instead, the facts would always render local_as information as a part of local_as_config as_number
            - This option has been deprecated and will be removed in a release after 2027-01-01.
            type: str
          local_as_config:
            description: Local Autonomous System Number options.
            type: dict
            suboptions:
              as_number:
                description:
                - Set Specify the local-as number for the eBGP neighbor.
                type: str
              no_prepend:
                description:
                - Do not prepend the local-as number to updates from the eBGP neighbor.
                type: bool
              replace_as:
                description:
                - Prepend only the local-as number to updates to eBGP neighbor.
                type: bool
              dual_as:
                description:
                - Connect using either the local-as number or the real as.
                type: bool
          log_neighbor_changes:
            description: Log message for neighbor up/down event.
            type: dict
            suboptions:
              set:
                description:
                - Set log-neighbor-changes.
                type: bool
              disable:
                description:
                - Disable logging of neighbor up/down event.
                type: bool
          low_memory:
            description: Behavior in low memory situations.
            type: dict
            suboptions:
              exempt:
                description: Do not shutdown this peer when under memory pressure.
                type: bool
          password:
            description: Configure a password for neighbor.
            type: dict
            suboptions:
              encryption:
                description:
                - 0 specifies an UNENCRYPTED neighbor password.
                - 3 specifies an 3DES ENCRYPTED neighbor password will follow.
                - 7 specifies a Cisco type 7  ENCRYPTED neighbor password will follow.
                type: int
              key:
                description: Authentication password.
                type: str
          path_attribute:
            description: BGP path attribute optional filtering.
            type: list
            elements: dict
            suboptions:
              action:
                description: Action.
                type: str
                choices: ["discard", "treat-as-withdraw"]
              type:
                description: Path attribute type
                type: int
              range:
                description: Path attribute range.
                type: dict
                suboptions:
                  start:
                    description: Path attribute range start value.
                    type: int
                  end:
                    description: Path attribute range end value.
                    type: int
          peer_type:
            description: Neighbor facing
            type: str
            choices: ["fabric-border-leaf", "fabric-external"]
          remote_as:
            description: Specify Autonomous System Number of the neighbor.
            type: str
          remote_as_route_map:
            description: Route-map to match prefix peer AS number.
            type: str
          remove_private_as:
            description: Remove private AS number from outbound updates.
            type: dict
            suboptions:
              set:
                description: Remove private AS.
                type: bool
              replace_as:
                description: Replace.
                type: bool
              all:
                description: All.
                type: bool
          shutdown:
            description: Administratively shutdown this neighbor.
            type: bool
          timers:
            description: Configure keepalive and hold timers.
            type: dict
            suboptions:
              keepalive:
                description: Keepalive interval (seconds).
                type: int
              holdtime:
                description: Holdtime (seconds).
                type: int
          transport:
            description: BGP transport connection.
            type: dict
            suboptions:
              connection_mode:
                description: Specify type of connection.
                type: dict
                suboptions:
                  passive:
                    description: Allow passive connection setup only.
                    type: bool
          ttl_security:
            description: Enable TTL Security Mechanism.
            type: dict
            suboptions:
              hops:
                description: Specify hop count for remote peer.
                type: int
          update_source:
            description: Specify source of BGP session and updates.
            type: str
      neighbor_down: &nbr_down
        description: Handle BGP neighbor down event, due to various reasons.
        type: dict
        suboptions:
          fib_accelerate:
            description: Accelerate the hardware updates for IP/IPv6 adjacencies for neighbor.
            type: bool
      nexthop:
        description: Nexthop resolution options.
        type: dict
        suboptions:
          suppress_default_resolution:
            description: Prohibit use of default route for nexthop address resolution.
            type: bool
      rd:
        description: Secondary Route Distinguisher for vxlan multisite border gateway.
        type: dict
        suboptions:
          dual:
            description: Generate Secondary RD for all VRFs and L2VNIs.
            type: bool
          id:
            description: Specify 2 byte value for ID.
            type: int
      reconnect_interval: &reconn_intv
        description: Configure connection reconnect interval.
        type: int
      router_id: &rtr_id
        description: Specify the IP address to use as router-id.
        type: str
      shutdown: &shtdwn
        description: Administratively shutdown BGP protocol.
        type: bool
      suppress_fib_pending: &suppr
        description: Advertise only routes that are programmed in hardware to peers.
        type: bool
      timers: &timers
        description: Configure bgp related timers.
        type: dict
        suboptions:
          bestpath_limit:
            description: Configure timeout for first bestpath after restart.
            type: dict
            suboptions:
              timeout:
                description: Bestpath timeout (seconds).
                type: int
              always:
                description: Configure update-delay-always option.
                type: bool
          bgp:
            description: Configure different bgp keepalive and holdtimes.
            type: dict
            suboptions:
              keepalive:
                description: Keepalive interval (seconds).
                type: int
              holdtime:
                description: Holdtime (seconds).
                type: int
          prefix_peer_timeout:
            description: Prefix Peer timeout (seconds).
            type: int
          prefix_peer_wait:
            description: Configure wait timer for a prefix peer.
            type: int
      vrfs:
        description: Virtual Router Context configurations.
        type: list
        elements: dict
        suboptions:
          vrf:
            description: VRF name.
            type: str
          allocate_index:
            description: Configure allocate-index.
            type: int
          bestpath: *bestpath
          cluster_id: *cluster_id
          confederation: *confederation
          graceful_restart: *graceful_restart
          local_as:
            description: Specify the local-as for this vrf.
            type: str
          log_neighbor_changes: *log_nbr
          maxas_limit: *maxas_limit
          neighbors: *nbr
          neighbor_down: *nbr_down
          reconnect_interval: *reconn_intv
          router_id: *rtr_id
          timers: *timers
  state:
    description:
    - The state the configuration should be left in.
    - State I(purged) removes all the BGP configurations from the
      target device. Use caution with this state.
    - State I(deleted) only removes BGP attributes that this modules
      manages and does not negate the BGP process completely. Thereby, preserving
      address-family related configurations under BGP context.
    - Running states I(deleted) and I(replaced) will result in an error if there
      are address-family configuration lines present under a neighbor,
      or a vrf context that is to be removed. Please use the
      M(cisco.nxos.nxos_bgp_af) or M(cisco.nxos.nxos_bgp_neighbor_af)
      modules for prior cleanup.
    - States C(merged) and C(replaced) will result in a failure if BGP is already configured
      with a different ASN than what is provided in the task. In such cases, please use
      state C(purged) to remove the existing BGP process and proceed further.
    - States C(replaced) and C(overridden) have the same behaviour for this module.
    - Refer to examples for more details.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - purged
    - parsed
    - gathered
    - rendered
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# Nexus9000v#

- name: Merge the provided configuration with the existing running configuration
  cisco.nxos.nxos_bgp_global:
    config:
      as_number: 65563
      router_id: 192.168.1.1
      bestpath:
        as_path:
          multipath_relax: true
        compare_neighborid: true
        cost_community_ignore: true
      confederation:
        identifier: 42
        peers:
          - 65020
          - 65030
          - 65040
      log_neighbor_changes: true
      maxas_limit: 20
      neighbors:
        - neighbor_address: 192.168.1.100
          neighbor_affinity_group:
            group_id: 160
          bmp_activate_server: 1
          remote_as: 65563
          description: NBR-1
          low_memory:
            exempt: true
        - neighbor_address: 192.168.1.101
          remote_as: 65563
          password:
            encryption: 7
            key: 12090404011C03162E
      neighbor_down:
        fib_accelerate: true
      vrfs:
        - vrf: site-1
          allocate_index: 5000
          local_as: 200
          log_neighbor_changes: true
          neighbors:
            - neighbor_address: 198.51.100.1
              description: site-1-nbr-1
              password:
                encryption: 3
                key: 13D4D3549493D2877B1DC116EE27A6BE
              remote_as: 65562
            - neighbor_address: 198.51.100.2
              remote_as: 65562
              description: site-1-nbr-2
        - vrf: site-2
          local_as: 300
          log_neighbor_changes: true
          neighbors:
            - neighbor_address: 203.0.113.2
              description: site-2-nbr-1
              password:
                encryption: 3
                key: AF92F4C16A0A0EC5BDF56CF58BC030F6
              remote_as: 65568
          neighbor_down:
            fib_accelerate: true

# Task output:
# ------------
# before: {}
#
# commands:
#  - router bgp 65563
#  - bestpath as-path multipath-relax
#  - bestpath compare-neighborid
#  - bestpath cost-community ignore
#  - confederation identifier 42
#  - log-neighbor-changes
#  - maxas-limit 20
#  - neighbor-down fib-accelerate
#  - router-id 192.168.1.1
#  - confederation peers 65020 65030 65040
#  - neighbor 192.168.1.100
#  - remote-as 65563
#  - affinity-group 160
#  - bmp-activate-server 1
#  - description NBR-1
#  - low-memory exempt
#  - neighbor 192.168.1.101
#  - remote-as 65563
#  - password 7 12090404011C03162E
#  - vrf site-1
#  - allocate-index 5000
#  - local-as 200
#  - log-neighbor-changes
#  - neighbor 198.51.100.1
#  - remote-as 65562
#  - description site-1-nbr-1
#  - password 3 13D4D3549493D2877B1DC116EE27A6BE
#  - neighbor 198.51.100.2
#  - remote-as 65562
#  - description site-1-nbr-2
#  - vrf site-2
#  - local-as 300
#  - log-neighbor-changes
#  - neighbor-down fib-accelerate
#  - neighbor 203.0.113.2
#  - remote-as 65568
#  - description site-2-nbr-1
#  - password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6
#
# after:
#    as_number: '65563'
#    bestpath:
#      as_path:
#        multipath_relax: true
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65040'
#    log_neighbor_changes: true
#    maxas_limit: 20
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    - neighbor_address: 192.168.1.101
#      password:
#        encryption: 7
#        key: 12090404011C03162E
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - allocate_index: 5000
#      local_as: '200'
#      log_neighbor_changes: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 198.51.100.1
#        password:
#          encryption: 3
#          key: 13D4D3549493D2877B1DC116EE27A6BE
#        remote_as: '65562'
#      - description: site-1-nbr-2
#        neighbor_address: 198.51.100.2
#        remote_as: '65562'
#      vrf: site-1
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - description: site-2-nbr-1
#        neighbor_address: 203.0.113.2
#        password:
#          encryption: 3
#          key: AF92F4C16A0A0EC5BDF56CF58BC030F6
#        remote_as: '65568'
#      vrf: site-2


# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65040
#   bestpath as-path multipath-relax
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 20
#   log-neighbor-changes
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   neighbor 192.168.1.101
#     remote-as 65563
#     password 7 12090404011C03162E
#   vrf site-1
#     local-as 200
#     log-neighbor-changes
#     allocate-index 5000
#     neighbor 198.51.100.1
#       remote-as 65562
#       description site-1-nbr-1
#       password 3 13D4D3549493D2877B1DC116EE27A6BE
#     neighbor 198.51.100.2
#       remote-as 65562
#       description site-1-nbr-2
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       remote-as 65568
#       description site-2-nbr-1
#       password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6

# Using replaced

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65040
#   bestpath as-path multipath-relax
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 20
#   log-neighbor-changes
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   neighbor 192.168.1.101
#     remote-as 65563
#     password 7 12090404011C03162E
#   vrf site-1
#     local-as 200
#     log-neighbor-changes
#     allocate-index 5000
#     neighbor 198.51.100.1
#       remote-as 65562
#       description site-1-nbr-1
#       password 3 13D4D3549493D2877B1DC116EE27A6BE
#     neighbor 198.51.100.2
#       remote-as 65562
#       description site-1-nbr-2
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       remote-as 65568
#       description site-2-nbr-1
#       password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6

- name: Replace BGP configuration with provided configuration
  cisco.nxos.nxos_bgp_global:
    config:
      as_number: 65563
      router_id: 192.168.1.1
      bestpath:
        compare_neighborid: true
        cost_community_ignore: true
      confederation:
        identifier: 42
        peers:
          - 65020
          - 65030
          - 65050
      maxas_limit: 40
      neighbors:
        - neighbor_address: 192.168.1.100
          neighbor_affinity_group:
            group_id: 160
          bmp_activate_server: 1
          remote_as: 65563
          description: NBR-1
          low_memory:
            exempt: true
      neighbor_down:
        fib_accelerate: true
      vrfs:
        - vrf: site-2
          local_as: 300
          log_neighbor_changes: true
          neighbors:
            - neighbor_address: 203.0.113.2
              password:
                encryption: 7
                key: 12090404011C03162E
          neighbor_down:
            fib_accelerate: true
    state: replaced

# Task output:
# ------------
#  before:
#    as_number: '65563'
#    bestpath:
#      as_path:
#        multipath_relax: true
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65040'
#    log_neighbor_changes: true
#    maxas_limit: 20
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    - neighbor_address: 192.168.1.101
#      password:
#        encryption: 7
#        key: 12090404011C03162E
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - allocate_index: 5000
#      local_as: '200'
#      log_neighbor_changes: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 198.51.100.1
#        password:
#          encryption: 3
#          key: 13D4D3549493D2877B1DC116EE27A6BE
#        remote_as: '65562'
#      - description: site-1-nbr-2
#        neighbor_address: 198.51.100.2
#        remote_as: '65562'
#      vrf: site-1
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - description: site-2-nbr-1
#        neighbor_address: 203.0.113.2
#        password:
#          encryption: 3
#          key: AF92F4C16A0A0EC5BDF56CF58BC030F6
#        remote_as: '65568'
#      vrf: site-2
#
# commands:
#  - router bgp 65563
#  - no bestpath as-path multipath-relax
#  - no log-neighbor-changes
#  - maxas-limit 40
#  - no confederation peers 65020 65030 65040
#  - confederation peers 65020 65030 65050
#  - no neighbor 192.168.1.101
#  - vrf site-2
#  - neighbor 203.0.113.2
#  - no remote-as 65568
#  - no description site-2-nbr-1
#  - password 7 12090404011C03162E
#  - no vrf site-1

#  after:
#    as_number: '65563'
#    bestpath:
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65050'
#    maxas_limit: 40
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - neighbor_address: 203.0.113.2
#        password:
#          encryption: 7
#          key: 12090404011C03162E
#      vrf: site-2
#
# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65050
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 40
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       password 7 12090404011C03162E

# Using deleted

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65040
#   bestpath as-path multipath-relax
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 20
#   log-neighbor-changes
#   address-family ipv4 unicast
#     default-metric 400
#     suppress-inactive
#     default-information originate
#   address-family ipv6 multicast
#     wait-igp-convergence
#     redistribute eigrp eigrp-1 route-map site-1-rmap
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   neighbor 192.168.1.101
#     remote-as 65563
#     password 7 12090404011C03162E
#   vrf site-1
#     local-as 200
#     log-neighbor-changes
#     allocate-index 5000
#     address-family ipv4 multicast
#       maximum-paths 40
#       dampen-igp-metric 1200
#     neighbor 198.51.100.1
#       remote-as 65562
#       description site-1-nbr-1
#       password 3 13D4D3549493D2877B1DC116EE27A6BE
#     neighbor 198.51.100.2
#       remote-as 65562
#       description site-1-nbr-2
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       remote-as 65568
#       description site-1-nbr-1
#       password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6

- name: Delete BGP configurations handled by this module
  cisco.nxos.nxos_bgp_global:
    state: deleted

# Task output:
# ------------

# before:
#    as_number: '65563'
#    bestpath:
#      as_path:
#        multipath_relax: true
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65040'
#    log_neighbor_changes: true
#    maxas_limit: 20
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    - neighbor_address: 192.168.1.101
#      password:
#        encryption: 7
#        key: 12090404011C03162E
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - allocate_index: 5000
#      local_as: '200'
#      log_neighbor_changes: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 198.51.100.1
#        password:
#          encryption: 3
#          key: 13D4D3549493D2877B1DC116EE27A6BE
#        remote_as: '65562'
#      - description: site-1-nbr-2
#        neighbor_address: 198.51.100.2
#        remote_as: '65562'
#      vrf: site-1
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 203.0.113.2
#        password:
#          encryption: 3
#          key: AF92F4C16A0A0EC5BDF56CF58BC030F6
#        remote_as: '65568'
#      vrf: site-2
#
# commands:
#   - router bgp 65563
#   - no bestpath as-path multipath-relax
#   - no bestpath compare-neighborid
#   - no bestpath cost-community ignore
#   - no confederation identifier 42
#   - no log-neighbor-changes
#   - no maxas-limit 20
#   - no neighbor-down fib-accelerate
#   - no router-id 192.168.1.1
#   - no confederation peers 65020 65030 65040
#   - no neighbor 192.168.1.100
#   - no neighbor 192.168.1.101
#   - no vrf site-1
#   - no vrf site-2
#
#  after:
#    as_number: '65563'
#
# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   address-family ipv4 unicast
#     default-metric 400
#     suppress-inactive
#     default-information originate
#   address-family ipv6 multicast
#     wait-igp-convergence
#     redistribute eigrp eigrp-1 route-map site-1-rmap
#

# Using purged

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65040
#   bestpath as-path multipath-relax
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 20
#   log-neighbor-changes
#   address-family ipv4 unicast
#     default-metric 400
#     suppress-inactive
#     default-information originate
#   address-family ipv6 multicast
#     wait-igp-convergence
#     redistribute eigrp eigrp-1 route-map site-1-rmap
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   neighbor 192.168.1.101
#     remote-as 65563
#     password 7 12090404011C03162E
#   vrf site-1
#     local-as 200
#     log-neighbor-changes
#     allocate-index 5000
#     address-family ipv4 multicast
#       maximum-paths 40
#       dampen-igp-metric 1200
#     neighbor 198.51.100.1
#       remote-as 65562
#       description site-1-nbr-1
#       password 3 13D4D3549493D2877B1DC116EE27A6BE
#     neighbor 198.51.100.2
#       remote-as 65562
#       description site-1-nbr-2
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       remote-as 65568
#       description site-1-nbr-1
#       password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6

- name: Purge all BGP configurations from the device
  cisco.nxos.nxos_bgp_global:
    state: purged

# Task output:
# ------------

# before:
#    as_number: '65563'
#    bestpath:
#      as_path:
#        multipath_relax: true
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65040'
#    log_neighbor_changes: true
#    maxas_limit: 20
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    - neighbor_address: 192.168.1.101
#      password:
#        encryption: 7
#        key: 12090404011C03162E
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - allocate_index: 5000
#      local_as: '200'
#      log_neighbor_changes: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 198.51.100.1
#        password:
#          encryption: 3
#          key: 13D4D3549493D2877B1DC116EE27A6BE
#        remote_as: '65562'
#      - description: site-1-nbr-2
#        neighbor_address: 198.51.100.2
#        remote_as: '65562'
#      vrf: site-1
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 203.0.113.2
#        password:
#          encryption: 3
#          key: AF92F4C16A0A0EC5BDF56CF58BC030F6
#        remote_as: '65568'
#      vrf: site-2
#
# commands:
#   - no router bgp 65563
#
#  after: {}
#
# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# Nexus9000v#

# Using rendered

- name: Render platform specific configuration lines (without connecting to the device)
  cisco.nxos.nxos_bgp_global:
    config:
      as_number: 65563
      router_id: 192.168.1.1
      bestpath:
        as_path:
          multipath_relax: true
        compare_neighborid: true
        cost_community_ignore: true
      confederation:
        identifier: 42
        peers:
          - 65020
          - 65030
          - 65040
      log_neighbor_changes: true
      maxas_limit: 20
      neighbors:
        - neighbor_address: 192.168.1.100
          neighbor_affinity_group:
            group_id: 160
          bmp_activate_server: 1
          remote_as: 65563
          description: NBR-1
          low_memory:
            exempt: true
        - neighbor_address: 192.168.1.101
          remote_as: 65563
          password:
            encryption: 7
            key: 12090404011C03162E
      neighbor_down:
        fib_accelerate: true
      vrfs:
        - vrf: site-1
          allocate_index: 5000
          local_as: 200
          log_neighbor_changes: true
          neighbors:
            - neighbor_address: 198.51.100.1
              description: site-1-nbr-1
              password:
                encryption: 3
                key: 13D4D3549493D2877B1DC116EE27A6BE
              remote_as: 65562
            - neighbor_address: 198.51.100.2
              remote_as: 65562
              description: site-1-nbr-2
        - vrf: site-2
          local_as: 300
          log_neighbor_changes: true
          neighbors:
            - neighbor_address: 203.0.113.2
              description: site-1-nbr-1
              password:
                encryption: 3
                key: AF92F4C16A0A0EC5BDF56CF58BC030F6
              remote_as: 65568
          neighbor_down:
            fib_accelerate: true

# Task output:
# ------------
# rendered:
#   - router bgp 65563
#   - bestpath as-path multipath-relax
#   - bestpath compare-neighborid
#   - bestpath cost-community ignore
#   - confederation identifier 42
#   - log-neighbor-changes
#   - maxas-limit 20
#   - neighbor-down fib-accelerate
#   - router-id 192.168.1.1
#   - confederation peers 65020 65030 65040
#   - neighbor 192.168.1.100
#   - remote-as 65563
#   - affinity-group 160
#   - bmp-activate-server 1
#   - description NBR-1
#   - low-memory exempt
#   - neighbor 192.168.1.101
#   - remote-as 65563
#   - password 7 12090404011C03162E
#   - vrf site-1
#   - allocate-index 5000
#   - local-as 200
#   - log-neighbor-changes
#   - neighbor 198.51.100.1
#   - remote-as 65562
#   - description site-1-nbr-1
#   - password 3 13D4D3549493D2877B1DC116EE27A6BE
#   - neighbor 198.51.100.2
#   - remote-as 65562
#   - description site-1-nbr-2
#   - vrf site-2
#   - local-as 300
#   - log-neighbor-changes
#   - neighbor-down fib-accelerate
#   - neighbor 203.0.113.2
#   - remote-as 65568
#   - description site-1-nbr-1
#   - password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6

# Using parsed

# parsed.cfg
# ------------
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65040
#   bestpath as-path multipath-relax
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 20
#   log-neighbor-changes
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   neighbor 192.168.1.101
#     remote-as 65563
#     password 7 12090404011C03162E
#   vrf site-1
#     local-as 200
#     log-neighbor-changes
#     allocate-index 5000
#     neighbor 198.51.100.1
#       remote-as 65562
#       description site-1-nbr-1
#       password 3 13D4D3549493D2877B1DC116EE27A6BE
#     neighbor 198.51.100.2
#       remote-as 65562
#       description site-1-nbr-2
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       remote-as 65568
#       description site-1-nbr-1
#       password 3 AF92F4C16A0A0EC5BDF56CF58BC030F6

- name: Parse externally provided BGP config
  cisco.nxos.nxos_bgp_global:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output:
# ------------
#  parsed:
#    as_number: '65563'
#    bestpath:
#      as_path:
#        multipath_relax: true
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65040'
#    log_neighbor_changes: true
#    maxas_limit: 20
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    - neighbor_address: 192.168.1.101
#      password:
#        encryption: 7
#        key: 12090404011C03162E
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - allocate_index: 5000
#      local_as: '200'
#      log_neighbor_changes: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 198.51.100.1
#        password:
#          encryption: 3
#          key: 13D4D3549493D2877B1DC116EE27A6BE
#        remote_as: '65562'
#      - description: site-1-nbr-2
#        neighbor_address: 198.51.100.2
#        remote_as: '65562'
#      vrf: site-1
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - description: site-1-nbr-1
#        neighbor_address: 203.0.113.2
#        password:
#          encryption: 3
#          key: AF92F4C16A0A0EC5BDF56CF58BC030F6
#        remote_as: '65568'
#      vrf: site-2

# Using gathered

# existing config
#
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65563
#   router-id 192.168.1.1
#   confederation identifier 42
#   confederation peers 65020 65030 65050
#   bestpath cost-community ignore
#   bestpath compare-neighborid
#   neighbor-down fib-accelerate
#   maxas-limit 40
#   neighbor 192.168.1.100
#     low-memory exempt
#     bmp-activate-server 1
#     remote-as 65563
#     description NBR-1
#     affinity-group 160
#   vrf site-1
#   vrf site-2
#     local-as 300
#     neighbor-down fib-accelerate
#     log-neighbor-changes
#     neighbor 203.0.113.2
#       password 7 12090404011C03162E

- name: Gather BGP facts using gathered
  cisco.nxos.nxos_bgp_global:
    state: gathered

# Task output:
# ------------
#  gathered:
#    as_number: '65563'
#    bestpath:
#      compare_neighborid: true
#      cost_community_ignore: true
#    confederation:
#      identifier: '42'
#      peers:
#      - '65020'
#      - '65030'
#      - '65050'
#    maxas_limit: 40
#    neighbor_down:
#      fib_accelerate: true
#    neighbors:
#    - bmp_activate_server: 1
#      description: NBR-1
#      low_memory:
#        exempt: true
#      neighbor_address: 192.168.1.100
#      neighbor_affinity_group:
#        group_id: 160
#      remote_as: '65563'
#    router_id: 192.168.1.1
#    vrfs:
#    - vrf: site-1
#    - local_as: '300'
#      log_neighbor_changes: true
#      neighbor_down:
#        fib_accelerate: true
#      neighbors:
#      - neighbor_address: 203.0.113.2
#        password:
#          encryption: 7
#          key: 12090404011C03162E
#      vrf: site-2

# Remove a neighbor having AF configurations with state replaced (will fail)

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   log-neighbor-changes
#   maxas-limit 20
#   router-id 198.51.100.2
#   neighbor 203.0.113.2
#     address-family ipv4 unicast
#       next-hop-self
#     remote-as 65538
#     affinity-group 160
#     description NBR-1
#     low-memory exempt
#   neighbor 192.0.2.1
#     remote-as 65537
#     password 7 12090404011C03162E

- name: Remove a neighbor having AF configurations (should fail)
  cisco.nxos.nxos_bgp_global:
    config:
      as_number: 65536
      router_id: 198.51.100.2
      maxas_limit: 20
      log_neighbor_changes: true
      neighbors:
        - neighbor_address: 192.0.2.1
          remote_as: 65537
          password:
            encryption: 7
            key: 12090404011C03162E
    state: replaced

# Task output:
# ------------
# fatal: [Nexus9000v]: FAILED! => changed=false
#    msg: Neighbor 203.0.113.2 has address-family configurations.
#         Please use the nxos_bgp_neighbor_af module to remove those first.

# Remove a VRF having AF configurations with state replaced (will fail)

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   log-neighbor-changes
#   maxas-limit 20
#   router-id 198.51.100.2
#   neighbor 192.0.2.1
#     remote-as 65537
#     password 7 12090404011C03162E
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#     neighbor 203.0.113.2
#       remote-as 65538
#       affinity-group 160
#       description NBR-1
#       low-memory exempt
#   vrf site-2
#     neighbor-down fib-accelerate

- name: Remove a VRF having AF configurations (should fail)
  cisco.nxos.nxos_bgp_global:
    config:
      as_number: 65536
      router_id: 198.51.100.2
      maxas_limit: 20
      log_neighbor_changes: true
      neighbors:
        - neighbor_address: 192.0.2.1
          remote_as: 65537
          password:
            encryption: 7
            key: 12090404011C03162E
      vrfs:
        - vrf: site-2
          neighbor_down:
            fib_accelerate: true
    state: replaced

# Task output:
# ------------
# fatal: [Nexus9000v]: FAILED! => changed=false
#    msg: VRF site-1 has address-family configurations.
#         Please use the nxos_bgp_af module to remove those first.
"""

RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
    - router bgp 65563
    - maxas-limit 20
    - router-id 192.168.1.1
    - confederation peers 65020 65030 65040
    - neighbor 192.168.1.100
    - remote-as 65563
    - affinity-group 160
    - bmp-activate-server 1
    - description NBR-1
    - low-memory exempt
    - vrf site-1
    - log-neighbor-changes
    - neighbor 198.51.100.1
    - remote-as 65562
    - description site-1-nbr-1
    - password 3 13D4D3549493D2877B1DC116EE27A6BE
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - router bgp 65563
    - maxas-limit 20
    - router-id 192.168.1.1
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.bgp_global.bgp_global import (
    Bgp_globalArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.bgp_global.bgp_global import (
    Bgp_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Bgp_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Bgp_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2016-2017, Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # OneView doc fragment
    DOCUMENTATION = r"""
options:
  config:
    description:
      - Path to a JSON configuration file containing the OneView client configuration. The configuration file is optional
        and when used should be present in the host running the ansible commands. If the file path is not provided, the configuration
        is loaded from environment variables. For links to example configuration files or how to use the environment variables
        verify the notes section.
    type: path
  api_version:
    description:
      - OneView API Version.
    type: int
  image_streamer_hostname:
    description:
      - IP address or hostname for the HPE Image Streamer REST API.
    type: str
  hostname:
    description:
      - IP address or hostname for the appliance.
    type: str
  username:
    description:
      - Username for API authentication.
    type: str
  password:
    description:
      - Password for API authentication.
    type: str

requirements:
  - Python >= 2.7.9

notes:
  - 'A sample configuration file for the config parameter can be found at:
    U(https://github.com/HewlettPackard/oneview-ansible/blob/master/examples/oneview_config-rename.json).'
  - 'Check how to use environment variables for configuration at: U(https://github.com/HewlettPackard/oneview-ansible#environment-variables).'
  - 'Additional Playbooks for the HPE OneView Ansible modules can be found at: U(https://github.com/HewlettPackard/oneview-ansible/tree/master/examples).'
  - 'The OneView API version used directly affects returned and expected fields in resources. Information on setting the desired
    API version and can be found at: U(https://github.com/HewlettPackard/oneview-ansible#setting-your-oneview-version).'
"""

    VALIDATEETAG = r"""
options:
  validate_etag:
    description:
      - When the ETag Validation is enabled, the request is conditionally processed only if the current ETag for the resource
        matches the ETag provided in the data.
    type: bool
    default: true
"""

    FACTSPARAMS = r"""
options:
  params:
    description:
      - List of parameters to delimit, filter and sort the list of resources.
      - 'Parameter keys allowed are:'
      - 'V(start): The first item to return, using 0-based indexing.'
      - 'V(count): The number of resources to return.'
      - 'V(filter): A general filter/query string to narrow the list of items returned.'
      - 'V(sort): The sort order of the returned data set.'
    type: dict
"""



# Copyright (c) 2024 Vladimir Botka <vbotka@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: iocage
short_description: C(iocage) inventory source
version_added: 10.2.0
author:
  - Vladimir Botka (@vbotka)
requirements:
  - iocage >= 1.8
description:
  - Get inventory hosts from the C(iocage) jail manager running on O(host).
  - By default, O(host) is V(localhost). If O(host) is not V(localhost) it is expected that the user running Ansible on the
    controller can connect to the O(host) account O(user) with SSH non-interactively and execute the command C(iocage list).
  - Uses a configuration file as an inventory source, it must end in C(.iocage.yml) or C(.iocage.yaml).
extends_documentation_fragment:
  - ansible.builtin.constructed
  - ansible.builtin.inventory_cache
options:
  plugin:
    description:
      - The name of this plugin, it should always be set to V(community.general.iocage) for this plugin to recognize it as
        its own.
    required: true
    choices: ['community.general.iocage']
    type: str
  host:
    description: The IP/hostname of the C(iocage) host.
    type: str
    default: localhost
  user:
    description:
      - C(iocage) user. It is expected that the O(user) is able to connect to the O(host) with SSH and execute the command
        C(iocage list). This option is not required if O(host=localhost).
    type: str
  sudo:
    description:
      - Enable execution as root.
      - This requires passwordless sudo of the command C(iocage list*).
    type: bool
    default: false
    version_added: 10.3.0
  sudo_preserve_env:
    description:
      - Preserve environment if O(sudo) is enabled.
      - This requires C(SETENV) sudoers tag.
    type: bool
    default: false
    version_added: 10.3.0
  get_properties:
    description:
      - Get jails' properties. Creates dictionary C(iocage_properties) for each added host.
    type: bool
    default: false
  env:
    description:
      - O(user)'s environment on O(host).
      - Enable O(sudo_preserve_env) if O(sudo) is enabled.
    type: dict
    default: {}
  hooks_results:
    description:
      - List of paths to the files in a jail.
      - Content of the files is stored in the items of the list C(iocage_hooks).
      - If a file is not available the item keeps the dash character C(-).
      - The variable C(iocage_hooks) is not created if O(hooks_results) is empty.
    type: list
    elements: path
    version_added: 10.4.0
  inventory_hostname_tag:
    description:
      - The name of the tag in the C(iocage properties notes) that contains the jails alias.
      - By default, the C(iocage list -l) column C(NAME) is used to name the jail.
      - This option requires the notes format C("t1=v1 t2=v2 ...").
      - The option O(get_properties) must be enabled.
    type: str
    version_added: 11.0.0
  inventory_hostname_required:
    description:
      - If enabled, the tag declared in O(inventory_hostname_tag) is required.
    type: bool
    default: false
    version_added: 11.0.0
notes:
  - You might want to test the command C(ssh user@host iocage list -l) on the controller before using this inventory plugin
    with O(user) specified and with O(host) other than V(localhost).
  - If you run this inventory plugin on V(localhost) C(ssh) is not used. In this case, test the command C(iocage list -l).
  - This inventory plugin creates variables C(iocage_*) for each added host.
  - The values of these variables are collected from the output of the command C(iocage list -l).
  - The names of these variables correspond to the output columns.
  - The column C(NAME) is used to name the added host.
  - The option O(hooks_results) expects the C(poolname) of a jail is mounted to C(/poolname). For example, if you activate
    the pool C(iocage) this plugin expects to find the O(hooks_results) items in the path C(/iocage/iocage/jails/<name>/root).
    If you mount the C(poolname) to a different path the easiest remedy is to create a symlink.
"""

EXAMPLES = r"""
---
# file name must end with iocage.yaml or iocage.yml
plugin: community.general.iocage
host: 10.1.0.73
user: admin

---
# user is not required if iocage is running on localhost (default)
plugin: community.general.iocage

---
# run cryptography without legacy algorithms
plugin: community.general.iocage
host: 10.1.0.73
user: admin
env:
  CRYPTOGRAPHY_OPENSSL_NO_LEGACY: 1

---
# execute as root
# sudoers example 'admin ALL=(ALL) NOPASSWD:SETENV: /usr/local/bin/iocage list*'
plugin: community.general.iocage
host: 10.1.0.73
user: admin
sudo: true
sudo_preserve_env: true
env:
  CRYPTOGRAPHY_OPENSSL_NO_LEGACY: 1

---
# enable cache
plugin: community.general.iocage
host: 10.1.0.73
user: admin
env:
  CRYPTOGRAPHY_OPENSSL_NO_LEGACY: 1
cache: true

---
# see inventory plugin ansible.builtin.constructed
plugin: community.general.iocage
host: 10.1.0.73
user: admin
env:
  CRYPTOGRAPHY_OPENSSL_NO_LEGACY: 1
cache: true
strict: false
compose:
  ansible_host: iocage_ip4
  release: iocage_release | split('-') | first
groups:
  test: inventory_hostname.startswith('test')
keyed_groups:
  - prefix: distro
    key: iocage_release
  - prefix: state
    key: iocage_state

---
# Read the file /var/db/dhclient-hook.address.epair0b in the jails and use it as ansible_host
plugin: community.general.iocage
host: 10.1.0.73
user: admin
hooks_results:
  - /var/db/dhclient-hook.address.epair0b
compose:
  ansible_host: iocage_hooks.0
groups:
  test: inventory_hostname.startswith('test')
"""

import os
import re
from subprocess import PIPE, Popen

from ansible.errors import AnsibleError, AnsibleParserError
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.plugins.inventory import BaseInventoryPlugin, Cacheable, Constructable
from ansible.utils.display import Display

display = Display()


def _parse_ip4(ip4):
    """Return dictionary iocage_ip4_dict. default = {ip4: [], msg: ''}.
    If item matches ifc|IP or ifc|CIDR parse ifc, ip, and mask.
    Otherwise, append item to msg.
    """

    iocage_ip4_dict = {}
    iocage_ip4_dict["ip4"] = []
    iocage_ip4_dict["msg"] = ""

    items = ip4.split(",")
    for item in items:
        if re.match("^\\w+\\|(?:\\d{1,3}\\.){3}\\d{1,3}.*$", item):
            i = re.split("\\||/", item)
            if len(i) == 3:
                iocage_ip4_dict["ip4"].append({"ifc": i[0], "ip": i[1], "mask": i[2]})
            else:
                iocage_ip4_dict["ip4"].append({"ifc": i[0], "ip": i[1], "mask": "-"})
        else:
            iocage_ip4_dict["msg"] += item

    return iocage_ip4_dict


class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
    """Host inventory parser for ansible using iocage as source."""

    NAME = "community.general.iocage"
    IOCAGE = "/usr/local/bin/iocage"

    def __init__(self):
        super().__init__()

    def verify_file(self, path):
        valid = False
        if super().verify_file(path):
            if path.endswith(("iocage.yaml", "iocage.yml")):
                valid = True
            else:
                self.display.vvv('Skipping due to inventory source not ending in "iocage.yaml" nor "iocage.yml"')
        return valid

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path)

        self._read_config_data(path)
        cache_key = self.get_cache_key(path)

        user_cache_setting = self.get_option("cache")
        attempt_to_read_cache = user_cache_setting and cache
        cache_needs_update = user_cache_setting and not cache

        if attempt_to_read_cache:
            try:
                results = self._cache[cache_key]
            except KeyError:
                cache_needs_update = True
        if not attempt_to_read_cache or cache_needs_update:
            results = self.get_inventory(path)
        if cache_needs_update:
            self._cache[cache_key] = results

        self.populate(results)

    def get_inventory(self, path):
        host = self.get_option("host")
        sudo = self.get_option("sudo")
        sudo_preserve_env = self.get_option("sudo_preserve_env")
        env = self.get_option("env")
        get_properties = self.get_option("get_properties")
        hooks_results = self.get_option("hooks_results")
        inventory_hostname_tag = self.get_option("inventory_hostname_tag")
        inventory_hostname_required = self.get_option("inventory_hostname_required")

        cmd = []
        my_env = os.environ.copy()
        if host == "localhost":
            my_env.update({str(k): str(v) for k, v in env.items()})
        else:
            user = self.get_option("user")
            cmd.append("ssh")
            cmd.append(f"{user}@{host}")
            cmd.extend([f"{k}={v}" for k, v in env.items()])

        cmd_list = cmd.copy()
        if sudo:
            cmd_list.append("sudo")
            if sudo_preserve_env:
                cmd_list.append("--preserve-env")
        cmd_list.append(self.IOCAGE)
        cmd_list.append("list")
        cmd_list.append("--long")
        try:
            p = Popen(cmd_list, stdout=PIPE, stderr=PIPE, env=my_env)
            stdout, stderr = p.communicate()
            if p.returncode != 0:
                raise AnsibleError(f"Failed to run cmd={cmd_list}, rc={p.returncode}, stderr={to_native(stderr)}")

            try:
                t_stdout = to_text(stdout, errors="surrogate_or_strict")
            except UnicodeError as e:
                raise AnsibleError(f"Invalid (non unicode) input returned: {e}") from e

        except Exception as e:
            raise AnsibleParserError(f"Failed to parse {to_native(path)}: {e}") from e

        results = {"_meta": {"hostvars": {}}}
        self.get_jails(t_stdout, results)

        if get_properties:
            for hostname in results["_meta"]["hostvars"]:
                cmd_get_properties = cmd.copy()
                cmd_get_properties.append(self.IOCAGE)
                cmd_get_properties.append("get")
                cmd_get_properties.append("--all")
                cmd_get_properties.append(f"{hostname}")
                try:
                    p = Popen(cmd_get_properties, stdout=PIPE, stderr=PIPE, env=my_env)
                    stdout, stderr = p.communicate()
                    if p.returncode != 0:
                        raise AnsibleError(
                            f"Failed to run cmd={cmd_get_properties}, rc={p.returncode}, stderr={to_native(stderr)}"
                        )

                    try:
                        t_stdout = to_text(stdout, errors="surrogate_or_strict")
                    except UnicodeError as e:
                        raise AnsibleError(f"Invalid (non unicode) input returned: {e}") from e

                except Exception as e:
                    raise AnsibleError(f"Failed to get properties: {e}") from e

                self.get_properties(t_stdout, results, hostname)

        if hooks_results:
            cmd_get_pool = cmd.copy()
            cmd_get_pool.append(self.IOCAGE)
            cmd_get_pool.append("get")
            cmd_get_pool.append("--pool")
            try:
                p = Popen(cmd_get_pool, stdout=PIPE, stderr=PIPE, env=my_env)
                stdout, stderr = p.communicate()
                if p.returncode != 0:
                    raise AnsibleError(
                        f"Failed to run cmd={cmd_get_pool}, rc={p.returncode}, stderr={to_native(stderr)}"
                    )
                try:
                    iocage_pool = to_text(stdout, errors="surrogate_or_strict").strip()
                except UnicodeError as e:
                    raise AnsibleError(f"Invalid (non unicode) input returned: {e}") from e
            except Exception as e:
                raise AnsibleError(f"Failed to get pool: {e}") from e

            for hostname in results["_meta"]["hostvars"]:
                iocage_hooks = []
                for hook in hooks_results:
                    path = f"/{iocage_pool}/iocage/jails/{hostname}/root{hook}"
                    cmd_cat_hook = cmd.copy()
                    cmd_cat_hook.append("cat")
                    cmd_cat_hook.append(path)
                    try:
                        p = Popen(cmd_cat_hook, stdout=PIPE, stderr=PIPE, env=my_env)
                        stdout, stderr = p.communicate()
                        if p.returncode != 0:
                            iocage_hooks.append("-")
                            continue

                        try:
                            iocage_hook = to_text(stdout, errors="surrogate_or_strict").strip()
                        except UnicodeError as e:
                            raise AnsibleError(f"Invalid (non unicode) input returned: {e}") from e

                    except Exception:
                        iocage_hooks.append("-")
                    else:
                        iocage_hooks.append(iocage_hook)

                results["_meta"]["hostvars"][hostname]["iocage_hooks"] = iocage_hooks

        # Optionally, get the jails names from the properties notes.
        # Requires the notes format "t1=v1 t2=v2 ..."
        if inventory_hostname_tag:
            if not get_properties:
                raise AnsibleError("Jail properties are needed to use inventory_hostname_tag. Enable get_properties")
            update = {}
            for hostname, host_vars in results["_meta"]["hostvars"].items():
                tags = dict(tag.split("=", 1) for tag in host_vars["iocage_properties"]["notes"].split() if "=" in tag)
                if inventory_hostname_tag in tags:
                    update[hostname] = tags[inventory_hostname_tag]
                elif inventory_hostname_required:
                    raise AnsibleError(f"Mandatory tag {inventory_hostname_tag!r} is missing in the properties notes.")
            for hostname, alias in update.items():
                results["_meta"]["hostvars"][alias] = results["_meta"]["hostvars"].pop(hostname)

        return results

    def get_jails(self, t_stdout, results):
        lines = t_stdout.splitlines()
        if len(lines) < 5:
            return
        indices = [i for i, val in enumerate(lines[1]) if val == "|"]
        for line in lines[3::2]:
            jail = [line[i + 1 : j].strip() for i, j in zip(indices[:-1], indices[1:])]
            iocage_name = jail[1]
            iocage_ip4_dict = _parse_ip4(jail[6])
            if iocage_ip4_dict["ip4"]:
                iocage_ip4 = ",".join([d["ip"] for d in iocage_ip4_dict["ip4"]])
            else:
                iocage_ip4 = "-"
            results["_meta"]["hostvars"][iocage_name] = {}
            results["_meta"]["hostvars"][iocage_name]["iocage_jid"] = jail[0]
            results["_meta"]["hostvars"][iocage_name]["iocage_boot"] = jail[2]
            results["_meta"]["hostvars"][iocage_name]["iocage_state"] = jail[3]
            results["_meta"]["hostvars"][iocage_name]["iocage_type"] = jail[4]
            results["_meta"]["hostvars"][iocage_name]["iocage_release"] = jail[5]
            results["_meta"]["hostvars"][iocage_name]["iocage_ip4_dict"] = iocage_ip4_dict
            results["_meta"]["hostvars"][iocage_name]["iocage_ip4"] = iocage_ip4
            results["_meta"]["hostvars"][iocage_name]["iocage_ip6"] = jail[7]
            results["_meta"]["hostvars"][iocage_name]["iocage_template"] = jail[8]
            results["_meta"]["hostvars"][iocage_name]["iocage_basejail"] = jail[9]

    def get_properties(self, t_stdout, results, hostname):
        properties = dict(x.split(":", 1) for x in t_stdout.splitlines())
        results["_meta"]["hostvars"][hostname]["iocage_properties"] = properties

    def populate(self, results):
        strict = self.get_option("strict")

        for hostname, host_vars in results["_meta"]["hostvars"].items():
            self.inventory.add_host(hostname, group="all")
            for var, value in host_vars.items():
                self.inventory.set_variable(hostname, var, value)
            self._set_composite_vars(self.get_option("compose"), host_vars, hostname, strict=True)
            self._add_host_to_composed_groups(self.get_option("groups"), host_vars, hostname, strict=strict)
            self._add_host_to_keyed_groups(self.get_option("keyed_groups"), host_vars, hostname, strict=strict)



# Copyright (c) 2016 Dag Wieers <dag@wieers.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: filetree
author: Dag Wieers (@dagwieers) <dag@wieers.com>
short_description: Recursively match all files in a directory tree
description:
  - This lookup enables you to template a complete tree of files on a target system while retaining permissions and ownership.
  - Supports directories, files and symlinks, including SELinux and other file properties.
  - If you provide more than one path, it implements a first_found logic, and does not process entries it already processed
    in previous paths. This enables merging different trees in order of importance, or add role_vars to specific paths to
    influence different instances of the same role.
options:
  _terms:
    description: Path(s) of files to read.
    required: true
    type: list
    elements: string
  exclude:
    description:
      - A regular expression string to exclude files. Applies to the base name of the file or directory.
    type: str
    version_added: 12.0.0
"""

EXAMPLES = r"""
- name: Create directories
  ansible.builtin.file:
    path: /web/{{ item.path }}
    state: directory
    mode: '{{ item.mode }}'
  with_community.general.filetree: web/
  when: item.state == 'directory'

- name: Template files (explicitly skip directories in order to use the 'src' attribute)
  ansible.builtin.template:
    src: '{{ item.src }}'
    # Your template files should be stored with a .j2 file extension,
    # but should not be deployed with it. splitext|first removes it.
    dest: /web/{{ item.path | splitext | first }}
    mode: '{{ item.mode }}'
  with_community.general.filetree: web/
  when: item.state == 'file'

- name: Recreate symlinks
  ansible.builtin.file:
    src: '{{ item.src }}'
    dest: /web/{{ item.path }}
    state: link
    follow: false  # avoid corrupting target files if the link already exists
    force: true
    mode: '{{ item.mode }}'
  with_community.general.filetree: web/
  when: item.state == 'link'

- name: list all files under web/
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.filetree', 'web/') }}"

- name: list all files under web/; exclude .git and .node_modules
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.filetree', 'web/', exclude='^(\.git|\.node_modules)$') }}
"""

RETURN = r"""
_raw:
  description: List of dictionaries with file information.
  type: list
  elements: dict
  contains:
    src:
      description:
        - Full path to file.
        - Not returned when RV(_raw[].state) is set to V(directory).
      type: path
    root:
      description: Allows filtering by original location.
      type: path
    path:
      description: Contains the relative path to root.
      type: path
    mode:
      description: The permissions the resulting file or directory.
      type: str
    state:
      description:
        - Type of the resulting file or directory.
        - V(directory) for a directory, V(file) for a regular file, V(link) for a symbolic link.
        - Other file types are skipped and not returned.
      type: str
      choices: [directory, file, link]
    owner:
      description: Name of the user that owns the file/directory.
      type: raw
    group:
      description: Name of the group that owns the file/directory.
      type: raw
    seuser:
      description: The user part of the SELinux file context.
      type: raw
    serole:
      description: The role part of the SELinux file context.
      type: raw
    setype:
      description: The type part of the SELinux file context.
      type: raw
    selevel:
      description: The level part of the SELinux file context.
      type: raw
    uid:
      description: Owner ID of the file/directory.
      type: int
    gid:
      description: Group ID of the file/directory.
      type: int
    size:
      description: Size of the target.
      type: int
    mtime:
      description: Time of last modification.
      type: float
    ctime:
      description: Time of last metadata update or creation (depends on OS).
      type: float
"""

import grp
import os
import pwd
import re
import stat

HAVE_SELINUX = False
try:
    import selinux

    HAVE_SELINUX = True
except ImportError:
    pass

from ansible.errors import AnsibleLookupError
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.plugins.lookup import LookupBase
from ansible.utils.display import Display

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

display = Display()


# If selinux fails to find a default, return an array of None
def selinux_context(path):
    context = [None, None, None, None]
    if HAVE_SELINUX and selinux.is_selinux_enabled():
        try:
            # note: the selinux module uses byte strings on python2 and text
            # strings on python3
            ret = selinux.lgetfilecon_raw(to_native(path))
        except OSError:
            return context
        if ret[0] != -1:
            # Limit split to 4 because the selevel, the last in the list,
            # may contain ':' characters
            context = ret[1].split(":", 3)
    return context


def file_props(root, path):
    """Returns dictionary with file properties, or return None on failure"""
    abspath = os.path.join(root, path)

    try:
        st = os.lstat(abspath)
    except OSError as e:
        display.warning(f"filetree: Error using stat() on path {abspath} ({e})")
        return None

    ret = dict(root=root, path=path)

    if stat.S_ISLNK(st.st_mode):
        ret["state"] = "link"
        ret["src"] = os.readlink(abspath)
    elif stat.S_ISDIR(st.st_mode):
        ret["state"] = "directory"
    elif stat.S_ISREG(st.st_mode):
        ret["state"] = "file"
        ret["src"] = abspath
    else:
        display.warning(f"filetree: Error file type of {abspath} is not supported")
        return None

    ret["uid"] = st.st_uid
    ret["gid"] = st.st_gid
    try:
        ret["owner"] = pwd.getpwuid(st.st_uid).pw_name
    except KeyError:
        ret["owner"] = st.st_uid
    try:
        ret["group"] = to_text(grp.getgrgid(st.st_gid).gr_name)
    except KeyError:
        ret["group"] = st.st_gid
    ret["mode"] = f"0{stat.S_IMODE(st.st_mode):03o}"
    ret["size"] = st.st_size
    ret["mtime"] = st.st_mtime
    ret["ctime"] = st.st_ctime

    if HAVE_SELINUX and selinux.is_selinux_enabled() == 1:
        context = selinux_context(abspath)
        ret["seuser"] = context[0]
        ret["serole"] = context[1]
        ret["setype"] = context[2]
        ret["selevel"] = context[3]

    return ret


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        basedir = self.get_basedir(variables)

        # Regular expression for exclude
        exclude = self.get_option("exclude")
        if exclude:
            try:
                exclude_pattern = re.compile(exclude)
            except re.error as e:
                raise AnsibleLookupError(f"Invalid exclude regular expression {exclude!r}: {e}") from e
        else:
            exclude_pattern = None

        ret = []
        for term in terms:
            term_file = os.path.basename(term)
            dwimmed_path = self._loader.path_dwim_relative(basedir, "files", os.path.dirname(term))
            path = os.path.join(dwimmed_path, term_file)
            display.debug(f"Walking '{path}'")
            for root, dirs, files in os.walk(path, topdown=True):
                # Filter files and directories using a regular expression
                if exclude_pattern is not None:
                    dirs[:] = [d for d in dirs if not exclude_pattern.match(d)]
                    files[:] = [f for f in files if not exclude_pattern.match(f)]

                for entry in dirs + files:
                    relpath = os.path.relpath(os.path.join(root, entry), path)

                    # Skip if relpath was already processed (from another root)
                    if relpath not in [entry["path"] for entry in ret]:
                        props = file_props(path, relpath)
                        if props is not None:
                            display.debug(f"  found '{os.path.join(path, relpath)}'")
                            ret.append(props)

        return ret



# Copyright (c) 2019, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

from ansible_collections.community.crypto.plugins.module_utils._crypto._objects_data import (
    OID_MAP,
)

OID_LOOKUP: dict[str, str] = {}
NORMALIZE_NAMES: dict[str, str] = {}
NORMALIZE_NAMES_SHORT: dict[str, str] = {}

for dotted, names in OID_MAP.items():
    for name in names:
        if name in NORMALIZE_NAMES and OID_LOOKUP[name] != dotted:
            raise AssertionError(  # pragma: no cover
                f'Name collision during setup: "{name}" for OIDs {dotted} and {OID_LOOKUP[name]}'
            )
        NORMALIZE_NAMES[name] = names[0]
        NORMALIZE_NAMES_SHORT[name] = names[-1]
        OID_LOOKUP[name] = dotted
for alias, original in [("userID", "userId")]:
    if alias in NORMALIZE_NAMES:
        raise AssertionError(  # pragma: no cover
            f'Name collision during adding aliases: "{alias}" (alias for "{original}") is already mapped to OID {OID_LOOKUP[alias]}'
        )
    NORMALIZE_NAMES[alias] = original
    NORMALIZE_NAMES_SHORT[alias] = NORMALIZE_NAMES_SHORT[original]
    OID_LOOKUP[alias] = OID_LOOKUP[original]


__all__ = ("NORMALIZE_NAMES", "NORMALIZE_NAMES_SHORT", "OID_LOOKUP")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for eos_route_maps
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: eos_route_maps
short_description: Manages Route Maps resource module
description: This module configures and manages the attributes of Route Mapd on Arista
  EOS platforms.
version_added: 2.1.0
author: Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
- Tested against Arista EOS 4.24.6F
- This module works with connection C(network_cli). See the L(EOS Platform Options,eos_platform_options).
options:
    config:
      description: A list of route-map options
      type: list
      elements: dict
      suboptions:
        route_map:
          description: Route map name.
          type: str
        entries:
          description: Route Map entries.
          type: list
          elements: dict
          suboptions:
            statement:
              description: statement name
              type: str
            source:
              description: Rename/Copy configuration
              type: dict
              suboptions:
                action:
                  description: rename or copy configuration
                  type: str
                  choices: ["rename", "copy"]
                source_map_name:
                  description: Source route map name.
                  type: str
                overwrite:
                  description: if true, overwrite existing config.
                  type: bool
            action:
              description: Action for matching routes
              type: str
              choices: ["deny", "permit"]
            sequence:
              description: Index in the sequence.
              type: int
            continue_sequence:
              description: Route map entry sequence number.
              type: int
            description:
              description: Description for the route map.
              type: str
            sub_route_map:
              description: Sub route map
              type: dict
              suboptions:
                name:
                  description: sub route map name
                  type: str
                invert_result:
                  description: Invert sub route map result
                  type: bool
            set:
              description: set route attributes.
              type: dict
              suboptions:
                as_path:
                  description: Set as-path.
                  type: dict
                  suboptions:
                    match:
                      description: Match the entire as-path.
                      type: dict
                      suboptions:
                        as_number:
                          description: as number to use (includes auto;in csv format)
                          type: str
                        none:
                          description: Remove matching AS numbers
                          type: bool
                    prepend:
                      description: Prepend to the as-path.
                      type: dict
                      suboptions:
                        as_number:
                          description: as number to prepend (includes auto;in csv format)
                          type: str
                        last_as:
                          description: The number of times to prepend the last AS number.
                          type: int
                bgp:
                  description: BGP AS path multipath weight.
                  type: int
                community_attributes:
                  description: BGP community attribute.
                  type: dict
                  suboptions:
                    graceful_shutdown:
                      description: Graceful shutdown
                      type: bool
                    community:
                      description: community attributes.
                      type: dict
                      suboptions:
                        number:
                          description: community number (in csv format).
                          type: str
                        list:
                          description: community list name.
                          type: str
                        graceful_shutdown:
                          description: Gracefully shutdown.
                          type: bool
                        additive:
                          description: Add to existing community.
                          type: bool
                        delete:
                          description: Delete matching communities.
                          type: bool
                        internet:
                          description: Internet community
                          type: bool
                        local_as:
                          description: Do not send outside local AS.
                          type: bool
                        no_advertise:
                          description: Do not advertise to any peer.
                          type: bool
                        no_export:
                          description: Do not export to next AS.
                          type: bool
                    none:
                      description: No community attribute.
                      type: bool
                distance:
                  description: Set protocol independent distance.
                  type: int
                evpn:
                  description: Keep the next hop when advertising to eBGP peers.
                  type: bool
                extcommunity:
                  description: BGP extended community attribute.
                  type: dict
                  suboptions:
                    lbw:
                      description: Link bandwith values.
                      type: dict
                      suboptions:
                        value:
                          description: Link Bandwidth extended community value.
                          type: str
                        aggregate:
                          description: Aggregate Link Bandwidth.
                          type: bool
                        divide:
                          description: Divide Link Bandwidth.
                          type: str
                          choices: ["equal", "ration"]
                    none:
                      description: No attribute.
                      type: bool
                    rt:
                      description: Route target extended community
                      type: dict
                      suboptions: &params01
                        vpn:
                          description: VPN extended community.
                          type: str
                        additive:
                          description: Add to the existing community.
                          type: bool
                        delete:
                          description: Delete matching communities.
                          type: bool
                    soo:
                      description: Site-of-Origin extended community.
                      type: dict
                      suboptions: *params01
                ip:
                  description: Set IP specific information.
                  type: dict
                  suboptions: &params02
                    address:
                      description: next hop address.
                      type: str
                    unchanged:
                      description: Keep the next hop when advertising to eBGP peer
                      type: bool
                    peer_address:
                      description: Use BGP peering addr as next-hop.
                      type: bool
                ipv6:
                  description: Set IPv6 specific information.
                  type: dict
                  suboptions: *params02
                isis_level:
                  description: IS-IS level.
                  type: str
                local_preference:
                  description: BGP local preference.
                  type: int
                metric:
                  description: Route metric.
                  type: dict
                  suboptions:
                    igp_param:
                      description: IGP parameter
                      type: str
                      choices: ['igp-metric', 'igp-nexthop-cost']
                    add:
                      description: Add igp-metric / igp-nexthop-cost
                      type: str
                      choices: ['igp-metric', 'igp-nexthop-cost']
                    value:
                      description: metric value to add or subtract(with +/- sign).
                      type: str
                metric_type:
                  description: Route metric type.
                  type: str
                  choices: ["type-1", "type-2"]
                nexthop:
                  description: Route next hop.
                  type: dict
                  suboptions:
                    value:
                      description: IGP metric value.
                      type: int
                    max_metric:
                      description: Set IGP max metric value.
                      type: bool
                origin:
                  description: Set bgp origin.
                  type: str
                  choices: ["egp", "igp", "incomplete"]
                segment_index:
                  description: MPLS Segment-routing Segment Index.
                  type: int
                tag:
                  description: Route tag
                  type: int
                weight:
                  description: BGP weight.
                  type: int
            match:
              description: Route map match rules.
              type: dict
              suboptions:
                aggregate_role: &params04
                  description: Role in BGP contributor-aggregate relation.
                  type: dict
                  suboptions:
                    contributor:
                      description: BGP aggregate's contributor.
                      type: bool
                    route_map:
                      description: Route map to apply against the aggregate route.
                      type: str
                as:
                  description: BGP AS number.
                  type: int
                as_path: &params05
                  description: Set as-path.
                  type: dict
                  suboptions:
                    path_list:
                      description: AS path list name.
                      type: str
                    length:
                      description: Specify as-path length ( with comparison operators like <= 60 and >= 40 ).
                      type: str
                community: &params06
                  description: BGP community attribute.
                  type: dict
                  suboptions:
                    community_list:
                      description: list of community names (in csv format).
                      type: str
                    exact_match:
                      description:  Do exact matching of communities.
                      type: bool
                    instances:
                      description: Match number of community instances ( with comparison operators like <= 60 and >= 40 ).
                      type: str
                extcommunity: &params07
                  description: extended community list name.
                  type: dict
                  suboptions:
                    community_list:
                      description: list of community names (in csv format).
                      type: str
                    exact_match:
                      description: Do exact matching of communities.
                      type: bool
                interface:
                  description: interface name.
                  type: str
                invert_result:
                  description: Invert match result.
                  type: dict
                  suboptions:
                    aggregate_role: *params04
                    as_path: *params05
                    community: *params06
                    extcommunity: *params07
                    large_community: *params07
                ip:
                  description: Set IP specific information.
                  type: dict
                  suboptions: &params08
                    address:
                      description: next hop destination.
                      type: dict
                      suboptions:
                        access_list:
                          description: ip access-list.
                          type: str
                        dynamic:
                          description: Configure dynamic prefix-list.
                          type: bool
                        prefix_list:
                          description: Prefix list.
                          type: str
                    next_hop:
                      description: next hop prefix list.
                      type: str
                    resolved_next_hop:
                      description: Route resolved prefix list.
                      type: str
                ipv6:
                  description: Set IPv6 specific information.
                  type: dict
                  suboptions: *params08
                large_community: *params07
                isis_level:
                  description: IS-IS level.
                  type: str
                local_preference:
                  description: BGP local preference.
                  type: int
                metric:
                  description: Route metric.
                  type: int
                metric_type:
                  description: Route metric type.
                  type: str
                  choices: ["type-1", "type-2"]
                route_type:
                  description: Route type
                  type: str
                router_id:
                  description: Router ID.
                  type: str
                source_protocol:
                  description: Source routing protocol,
                  type: str
                tag:
                  description: Route tag
                  type: int
    running_config:
      description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the EOS device by
        executing the command B(show running-config | section route-map).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
      type: str
    state:
      description:
      - The state the configuration should be left in.
      type: str
      choices:
      - deleted
      - merged
      - overridden
      - replaced
      - gathered
      - rendered
      - parsed
      default: merged
"""
EXAMPLES = """
# Using merged
# Before state
# veos#show running-config | section route-map
# veos#

- name: Merge provided configuration with device configuration
  arista.eos.eos_route_maps:
    config:
      - route_map: "mapmerge"
        entries:
          - description: "merged_map"
            action: "permit"
            sequence: 10
            match:
              router_id: 22
          - description: "newmap"
            action: "deny"
            sequence: 25
            continue_sequence: 45
            match:
              interface: "Ethernet1"
      - route_map: "mapmerge2"
        entries:
          - sub_route_map:
              name: "mapmerge"
            action: "deny"
            sequence: 45
            set:
              metric:
                value: 25
                add: "igp-metric"
              as_path:
                prepend:
                  last_as: 2
            match:
              ipv6:
                resolved_next_hop: "list1"
    state: merged

# After State:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    description merged_map
#    match router-id prefix-list 22
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# !
# route-map test permit 10
# veos#

# Module Execution:

#  "after": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "description": "merged_map",
#                     "match": {
#                         "router_id": "22"
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_sequence": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ],
#     "before": {},
#     "changed": true,
#     "commands": [
#         "route-map mapmerge permit 10",
#         "match router-id prefix-list 22",
#         "description merged_map",
#         "route-map mapmerge deny 25",
#         "match interface Ethernet1",
#         "description newmap",
#         "continue 45",
#         "route-map mapmerge2 deny 45",
#         "match ipv6 resolved-next-hop prefix-list list1",
#         "set metric 25 +igp-metric",
#         "set as-path prepend last-as 2",
#         "sub-route-map mapmerge"
#     ],
#

# Using replaced:

# Before State:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    description merged_map
#    match router-id prefix-list 22
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# !
# veos#

- name: Replace
  arista.eos.eos_route_maps:
    config:
      - route_map: "mapmerge"
        entries:
          - action: "permit"
            sequence: 10
            match:
              ipv6:
                resolved_next_hop: "listr"
          - action: "deny"
            sequence: 90
            set:
              extcommunity:
                rt:
                  vpn: "22:11"
                  delete: true
              ip:
                unchanged: true
    state: replaced

# After State:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    match ipv6 resolved-next-hop prefix-list listr
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge deny 90
#    set ip next-hop unchanged
#    set extcommunity rt 22:11 delete
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# !
#
# Module Execution:
#
#     "after": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "listr"
#                         }
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_sequence": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 },
#                 {
#                     "action": "deny",
#                     "sequence": 90,
#                     "set": {
#                         "extcommunity": {
#                             "rt": {
#                                 "delete": true,
#                                 "vpn": "22:11"
#                             }
#                         },
#                         "ip": {
#                             "unchanged": true
#                         }
#                     }
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "sequence": 10
#                 }
#             ],
#             "route_map": "test"
#         }
#     ],
#     "before": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "description": "merged_map",
#                     "match": {
#                         "router_id": "22"
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_sequence": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "route-map mapmerge permit 10",
#         "match ipv6 resolved-next-hop prefix-list listr",
#         "no match router-id prefix-list 22",
#         "no description",
#         "route-map mapmerge deny 90",
#         "set extcommunity rt 22:11 delete",
#         "set ip next-hop unchanged"
#     ],
#
#
# Using Overridden:

# Before state:
# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    match ipv6 resolved-next-hop prefix-list listr
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge deny 90
#    set ip next-hop unchanged
#    set extcommunity rt 22:11 delete
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# !
# route-map test permit 10
# veos#

- name: Override
  arista.eos.eos_route_maps:
    config:
      - route_map: "mapmerge"
        entries:
          - action: "permit"
            sequence: 10
            match:
              ipv6:
                resolved_next_hop: "listr"
          - action: "deny"
            sequence: 90
            set:
              metric:
                igp_param: "igp-nexthop-cost"
    state: overridden

# After State:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    match ipv6 resolved-next-hop prefix-list listr
# !
# route-map mapmerge deny 90
#    set metric igp-nexthop-cost
# veos#
#
#
#    "after": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "listr"
#                         }
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "sequence": 90,
#                     "set": {
#                         "metric": {
#                             "igp_param": "igp-nexthop-cost"
#                         }
#                     }
#                 }
#             ],
#             "route_map": "mapmerge"
#         }
#     ],
#     "before": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "listr"
#                         }
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_sequence": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 },
#                 {
#                     "action": "deny",
#                     "sequence": 90,
#                     "set": {
#                         "extcommunity": {
#                             "rt": {
#                                 "delete": true,
#                                 "vpn": "22:11"
#                             }
#                         },
#                         "ip": {
#                             "unchanged": true
#                         }
#                     }
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "sequence": 10
#                 }
#             ],
#             "route_map": "test"
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "no route-map mapmerge deny 25",
#         "no route-map mapmerge2 deny 45",
#         "no route-map test permit 10",
#         "route-map mapmerge deny 90",
#         "set metric igp-nexthop-cost",
#         "no set ip next-hop unchanged",
#         "no set extcommunity rt 22:11 delete"
#     ],
#
# Using deleted:
# Before State:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    description merged_map
#    match router-id prefix-list 22
#    match ipv6 resolved-next-hop prefix-list listr
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge deny 90
#    set metric igp-nexthop-cost
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# veos#

- name: Delete route-map
  arista.eos.eos_route_maps:
    config:
      - route_map: "mapmerge"
    state: deleted
  become: true
  tags:
    - deleted1

# After State:

# veos#show running-config | section route-map
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# veos#
#
# Module Execution:
#
# "after": [
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ],
#     "before": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "description": "merged_map",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "listr"
#                         },
#                         "router_id": "22"
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 },
#                 {
#                     "action": "deny",
#                     "sequence": 90,
#                     "set": {
#                         "metric": {
#                             "igp_param": "igp-nexthop-cost"
#                         }
#                     }
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "no route-map mapmerge"
#     ],

# Using deleted to delete all route-maps:

# Before State:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    description merged_map
#    match router-id prefix-list 22
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# veos#

- name: Delete all route-maps
  arista.eos.eos_route_maps:
    state: deleted

# After State:
# ------------

# veos#show running-config | section route-map
# veos#
#
# Module Execution:
#
# "after": {},
#     "before": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "description": "merged_map",
#                     "match": {
#                         "router_id": "22"
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "no route-map mapmerge",
#         "no route-map mapmerge2"
#     ],

# Using gathered:

# Device configs:

# veos#show running-config | section route-map
# route-map mapmerge permit 10
#    description merged_map
#    match router-id prefix-list 22
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2
# veos#

- name: gather configs
  arista.eos.eos_route_maps:
    state: gathered

# Module Execution:
#   "gathered": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "description": "merged_map",
#                     "match": {
#                         "router_id": "22"
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_sequence": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ],

# Using rendered:

- name: Render provided configuration
  arista.eos.eos_route_maps:
    config:
      - route_map: "mapmerge"
        entries:
          - description: "merged_map"
            action: "permit"
            sequence: 10
            match:
              router_id: 22
            set:
              bgp: 20
          - description: "newmap"
            action: "deny"
            sequence: 25
            continue_sequence: 45
            match:
              interface: "Ethernet1"
      - route_map: "mapmerge2"
        entries:
          - sub_route_map:
              name: "mapmerge"
            action: "deny"
            sequence: 45
            set:
              metric:
                value: 25
                add: "igp-metric"
              as_path:
                prepend:
                  last_as: 2
            match:
              ipv6:
                resolved_next_hop: "list1"
    state: rendered

# Task output:
# ------------

# "rendered": [
#         "route-map mapmerge permit 10",
#         "match router-id prefix-list 22",
#         "set bgp bestpath as-path weight 20",
#         "description merged_map",
#         "route-map mapmerge deny 25",
#         "match interface Ethernet1",
#         "description newmap",
#         "continue 45",
#         "route-map mapmerge2 deny 45",
#         "match ipv6 resolved-next-hop prefix-list list1",
#         "set metric 25 +igp-metric",
#         "set as-path prepend last-as 2",
#         "sub-route-map mapmerge"
#     ]

# Using parsed:

# parsed.cfg
# route-map mapmerge permit 10
#    description merged_map
#    match router-id prefix-list 22
#    set bgp bestpath as-path weight 20
# !
# route-map mapmerge deny 25
#    description newmap
#    match interface Ethernet1
#    continue 45
# !
# route-map mapmerge2 deny 45
#    match ipv6 resolved-next-hop prefix-list list1
#    sub-route-map mapmerge
#    set metric 25 +igp-metric
#    set as-path prepend last-as 2

- name: parse configs
  arista.eos.eos_route_maps:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# Module Execution:
# "parsed": [
#         {
#             "entries": [
#                 {
#                     "action": "permit",
#                     "description": "merged_map",
#                     "match": {
#                         "router_id": "22"
#                     },
#                     "sequence": 10,
#                     "set": {
#                         "bgp": 20
#                     }
#                 },
#                 {
#                     "action": "deny",
#                     "continue_sequence": 45,
#                     "description": "newmap",
#                     "match": {
#                         "interface": "Ethernet1"
#                     },
#                     "sequence": 25
#                 }
#             ],
#             "route_map": "mapmerge"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "resolved_next_hop": "list1"
#                         }
#                     },
#                     "sequence": 45,
#                     "set": {
#                         "as_path": {
#                             "prepend": {
#                                 "last_as": 2
#                             }
#                         },
#                         "metric": {
#                             "add": "igp-metric",
#                             "value": "25"
#                         }
#                     },
#                     "sub_route_map": {
#                         "name": "mapmerge"
#                     }
#                 }
#             ],
#             "route_map": "mapmerge2"
#         }
#     ]
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.route_maps.route_maps import (
    Route_mapsArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.route_maps.route_maps import (
    Route_maps,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Route_mapsArgs.argument_spec,
        mutually_exclusive=[],
        required_if=[],
        supports_check_mode=False,
    )

    result = Route_maps(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2024, Florian Apolloner (@apollo13)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_acl_bootstrap
short_description: Bootstrap ACLs in Consul
version_added: 8.3.0
description:
  - Allows bootstrapping of ACLs in a Consul cluster, see U(https://developer.hashicorp.com/consul/api-docs/acl#bootstrap-acls)
    for details.
author:
  - Florian Apolloner (@apollo13)
extends_documentation_fragment:
  - community.general._consul
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    description:
      - Whether the token should be present or absent.
    choices: ['present', 'bootstrapped']
    default: present
    type: str
  bootstrap_secret:
    description:
      - The secret to be used as secret ID for the initial token.
      - Needs to be an UUID.
    type: str
"""

EXAMPLES = r"""
- name: Bootstrap the ACL system
  community.general.consul_acl_bootstrap:
    bootstrap_secret: 22eaeed1-bdbd-4651-724e-42ae6c43e387
"""

RETURN = r"""
result:
  description:
    - The bootstrap result as returned by the Consul HTTP API.
    - B(Note:) If O(bootstrap_secret) has been specified the C(SecretID) and C(ID) do not contain the secret but C(VALUE_SPECIFIED_IN_NO_LOG_PARAMETER).
      If you pass O(bootstrap_secret), make sure your playbook/role does not depend on this return value!
  returned: changed
  type: dict
  sample:
    AccessorID: 834a5881-10a9-a45b-f63c-490e28743557
    CreateIndex: 25
    CreateTime: '2024-01-21T20:26:27.114612038+01:00'
    Description: Bootstrap Token (Global Management)
    Hash: X2AgaFhnQGRhSSF/h0m6qpX1wj/HJWbyXcxkEM/5GrY=
    ID: VALUE_SPECIFIED_IN_NO_LOG_PARAMETER
    Local: false
    ModifyIndex: 25
    Policies:
      - ID: 00000000-0000-0000-0000-000000000001
        Name: global-management
    SecretID: VALUE_SPECIFIED_IN_NO_LOG_PARAMETER
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._consul import (
    AUTH_ARGUMENTS_SPEC,
    RequestError,
    _ConsulModule,
)

_ARGUMENT_SPEC = {
    "state": dict(type="str", choices=["present", "bootstrapped"], default="present"),
    "bootstrap_secret": dict(type="str", no_log=True),
}
_ARGUMENT_SPEC.update(AUTH_ARGUMENTS_SPEC)
_ARGUMENT_SPEC.pop("token")


def main():
    module = AnsibleModule(_ARGUMENT_SPEC)
    consul_module = _ConsulModule(module)

    data = {}
    if "bootstrap_secret" in module.params:
        data["BootstrapSecret"] = module.params["bootstrap_secret"]

    try:
        response = consul_module.put("acl/bootstrap", data=data)
    except RequestError as e:
        if e.status == 403 and b"ACL bootstrap no longer allowed" in e.response_data:
            return module.exit_json(changed=False)
        raise
    else:
        return module.exit_json(changed=True, result=response)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_fibre_channel_zone_policy
short_description: Manage Fibre Channel Zone Policies for Cisco Intersight
description:
  - Create, update, and delete Fibre Channel Zone Policies on Cisco Intersight.
  - Manage FC target members (zones) within FC zone policies.
  - FC Zone policies define zoning configurations for Fibre Channel networks.
  - Supports SIST (Single Initiator Single Target), SIMT (Single Initiator Multiple Target), and None zoning types.
  - When zoning type is None, targets cannot be added and existing targets will be disabled.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/FcZonePolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Fibre Channel Zone Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Fibre Channel Zone Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  fc_target_zoning_type:
    description:
      - Type of FC zoning configuration.
      - C(SIST) - Single Initiator Single Target zoning.
      - C(SIMT) - Single Initiator Multiple Target zoning.
      - C(None) - No zoning. When None is selected, targets cannot be added and existing targets will be disabled.
    type: str
    choices: [SIST, SIMT, None]
    default: None
  fc_target_members:
    description:
      - List of FC target members (zones) to be created within the FC zone policy.
      - Each target defines a WWPN that is a member of the FC zone.
      - Leave empty when fc_target_zoning_type is None.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name identifier for the FC target member.
          - Must be unique within the policy.
        type: str
        required: true
      wwpn:
        description:
          - World Wide Port Name (WWPN) that is a member of the FC zone.
          - Format should be colon-separated hex values (e.g., 21:00:00:e0:8b:05:05:04).
          - Must be a valid WWPN format.
        type: str
        required: true
      switch_id:
        description:
          - Unique identifier for the Fabric object.
          - Specifies which fabric interconnect the target is associated with.
        type: str
        choices: [A, B]
        default: A
      vsan_id:
        description:
          - VSAN ID with scope defined as Storage in the VSAN policy.
          - Must be between 1 and 4093.
        type: int
        required: true
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create FC Zone Policy with SIMT zoning and multiple targets
  cisco.intersight.intersight_fibre_channel_zone_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-zone-policy-simt"
    description: "FC Zone policy with SIMT zoning"
    fc_target_zoning_type: SIMT
    fc_target_members:
      - name: "target1"
        wwpn: "21:00:00:e0:8b:05:05:04"
        switch_id: A
        vsan_id: 100
      - name: "target2"
        wwpn: "21:00:00:e0:8b:05:05:03"
        switch_id: B
        vsan_id: 100
      - name: "target3"
        wwpn: "21:00:00:e0:8b:05:05:09"
        switch_id: A
        vsan_id: 200
    tags:
      - Key: Environment
        Value: Production
    state: present

- name: Create FC Zone Policy with SIST zoning
  cisco.intersight.intersight_fibre_channel_zone_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-zone-policy-sist"
    description: "FC Zone policy with SIST zoning"
    fc_target_zoning_type: SIST
    fc_target_members:
      - name: "single-target"
        wwpn: "21:00:00:e0:8b:05:05:01"
        switch_id: A
        vsan_id: 100
    state: present

- name: Create FC Zone Policy with no zoning (None type)
  cisco.intersight.intersight_fibre_channel_zone_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-zone-policy-none"
    description: "FC Zone policy with no zoning"
    fc_target_zoning_type: None
    state: present

- name: Update FC Zone Policy - add new target
  cisco.intersight.intersight_fibre_channel_zone_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-zone-policy-simt"
    description: "Updated FC Zone policy"
    fc_target_zoning_type: SIMT
    fc_target_members:
      - name: "target1"
        wwpn: "21:00:00:e0:8b:05:05:04"
        switch_id: A
        vsan_id: 100
      - name: "target2"
        wwpn: "21:00:00:e0:8b:05:05:03"
        switch_id: B
        vsan_id: 100
      - name: "target3"
        wwpn: "21:00:00:e0:8b:05:05:09"
        switch_id: A
        vsan_id: 200
      - name: "target4"
        wwpn: "21:00:00:e0:8b:05:05:10"
        switch_id: B
        vsan_id: 200
    state: present

- name: Update FC Zone Policy - remove a target (only specify remaining targets)
  cisco.intersight.intersight_fibre_channel_zone_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-zone-policy-simt"
    fc_target_zoning_type: SIMT
    fc_target_members:
      - name: "target1"
        wwpn: "21:00:00:e0:8b:05:05:04"
        switch_id: A
        vsan_id: 100
      - name: "target3"
        wwpn: "21:00:00:e0:8b:05:05:09"
        switch_id: A
        vsan_id: 200
    state: present

- name: Delete FC Zone Policy
  cisco.intersight.intersight_fibre_channel_zone_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-zone-policy-simt"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "fc-zone-policy-simt",
        "ObjectType": "fabric.FcZonePolicy",
        "Moid": "1234567890abcdef12345678",
        "Description": "FC Zone policy with SIMT zoning",
        "FcTargetZoningType": "SIMT",
        "FcTargetMembers": [
            {
                "Name": "target1",
                "Wwpn": "21:00:00:e0:8b:05:05:04",
                "SwitchId": "A",
                "VsanId": 100
            },
            {
                "Name": "target2",
                "Wwpn": "21:00:00:e0:8b:05:05:03",
                "SwitchId": "B",
                "VsanId": 100
            }
        ],
        "Organization": {
            "Moid": "abcdef1234567890abcdef12",
            "ObjectType": "organization.Organization"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec
import re


def validate_wwpn(wwpn):
    """
    Validate WWPN format.
    """
    wwpn_pattern = r'^([0-9a-fA-F]{2}:){7}[0-9a-fA-F]{2}$'
    if not re.match(wwpn_pattern, wwpn):
        raise ValueError(f"Invalid WWPN format '{wwpn}'. Expected format: XX:XX:XX:XX:XX:XX:XX:XX (hex values)")
    return True


def validate_vsan_id(vsan_id):
    """
    Validate VSAN ID is within acceptable range (1-4093).
    """
    if vsan_id < 1 or vsan_id > 4093:
        raise ValueError(f"VSAN ID {vsan_id} is out of valid range (1-4093)")
    return True


def validate_parameters(module):
    """
    Validate module parameters for FC Zone policy configuration.
    """
    if module.params['state'] != 'present':
        return
    fc_target_zoning_type = module.params.get('fc_target_zoning_type')
    fc_target_members = module.params.get('fc_target_members', [])
    # Validate that if zoning type is None, no targets should be provided
    if fc_target_zoning_type == 'None' and fc_target_members:
        module.fail_json(msg="When fc_target_zoning_type is 'None', fc_target_members should be empty. Targets cannot be added when zoning type is None.")
    # Validate fc_target_members if provided
    if fc_target_members:
        for target in fc_target_members:
            # Validate WWPN format
            wwpn = target.get('wwpn')
            if wwpn:
                try:
                    validate_wwpn(wwpn)
                except ValueError as e:
                    module.fail_json(msg=str(e))
            # Validate VSAN ID range
            vsan_id = target.get('vsan_id')
            if vsan_id:
                try:
                    validate_vsan_id(vsan_id)
                except ValueError as e:
                    module.fail_json(msg=str(e))


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        fc_target_zoning_type=dict(type='str', choices=['SIST', 'SIMT', 'None'], default='None'),
        fc_target_members=dict(type='list', elements='dict', options=dict(
            name=dict(type='str', required=True),
            wwpn=dict(type='str', required=True),
            switch_id=dict(type='str', choices=['A', 'B'], default='A'),
            vsan_id=dict(type='int', required=True)
        ))
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)

    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Validate module parameters
    validate_parameters(module)

    # Resource path used to configure policy
    resource_path = '/fabric/FcZonePolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.api_body['FcTargetZoningType'] = module.params['fc_target_zoning_type']
        # Build FcTargetMembers list
        fc_target_members = []
        if module.params.get('fc_target_members'):
            for target in module.params['fc_target_members']:
                fc_target_members.append({
                    'Name': target['name'],
                    'Wwpn': target['wwpn'],
                    'SwitchId': target['switch_id'],
                    'VsanId': target['vsan_id']
                })
        intersight.api_body['FcTargetMembers'] = fc_target_members
        intersight.set_tags_and_description()

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_swarm
short_description: Manage Swarm cluster
description:
  - Create a new Swarm cluster.
  - Add/Remove nodes or managers to an existing cluster.
extends_documentation_fragment:
  - community.docker._docker
  - community.docker._docker.docker_py_2_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  idempotent:
    support: full

options:
  advertise_addr:
    description:
      - Externally reachable address advertised to other nodes.
      - This can either be an address/port combination in the form V(192.168.1.1:4567), or an interface followed by a port
        number, like V(eth0:4567).
      - If the port number is omitted, the port number from the listen address is used.
      - If O(advertise_addr) is not specified, it will be automatically detected when possible.
      - Only used when swarm is initialised or joined. Because of this it is not considered for idempotency checking.
    type: str
  default_addr_pool:
    description:
      - Default address pool in CIDR format.
      - Only used when swarm is initialised. Because of this it is not considered for idempotency checking.
      - Requires API version >= 1.39.
    type: list
    elements: str
  subnet_size:
    description:
      - Default address pool subnet mask length.
      - Only used when swarm is initialised. Because of this it is not considered for idempotency checking.
      - Requires API version >= 1.39.
    type: int
  listen_addr:
    description:
      - Listen address used for inter-manager communication.
      - This can either be an address/port combination in the form V(192.168.1.1:4567), or an interface followed by a port
        number, like V(eth0:4567).
      - If the port number is omitted, the default swarm listening port is used.
      - Only used when swarm is initialised or joined. Because of this it is not considered for idempotency checking.
    type: str
    default: 0.0.0.0:2377
  force:
    description:
      - Use with state V(present) to force creating a new Swarm, even if already part of one.
      - Use with state V(absent) to Leave the swarm even if this node is a manager.
    type: bool
    default: false
  state:
    description:
      - Set to V(present), to create/update a new cluster.
      - Set to V(join), to join an existing cluster.
      - Set to V(absent), to leave an existing cluster.
      - Set to V(remove), to remove an absent node from the cluster. Note that removing requires Docker SDK for Python >=
        2.4.0.
      - M(community.docker.docker_node) can be used to demote a manager before removal.
    type: str
    default: present
    choices:
      - present
      - join
      - absent
      - remove
  node_id:
    description:
      - Swarm id of the node to remove.
      - Used with O(state=remove).
    type: str
  join_token:
    description:
      - Swarm token used to join a swarm cluster.
      - Used with O(state=join).
      - If this value is specified, the corresponding value in the return values will be censored by Ansible. This is a side-effect
        of this value not being logged.
    type: str
  remote_addrs:
    description:
      - Remote address of one or more manager nodes of an existing Swarm to connect to.
      - Used with O(state=join).
    type: list
    elements: str
  task_history_retention_limit:
    description:
      - Maximum number of tasks history stored.
      - Docker default value is V(5).
    type: int
  snapshot_interval:
    description:
      - Number of logs entries between snapshot.
      - Docker default value is V(10000).
    type: int
  keep_old_snapshots:
    description:
      - Number of snapshots to keep beyond the current snapshot.
      - Docker default value is V(0).
    type: int
  log_entries_for_slow_followers:
    description:
      - Number of log entries to keep around to sync up slow followers after a snapshot is created.
    type: int
  heartbeat_tick:
    description:
      - Amount of ticks (in seconds) between each heartbeat.
      - Docker default value is V(1) seconds.
    type: int
  election_tick:
    description:
      - Amount of ticks (in seconds) needed without a leader to trigger a new election.
      - Docker default value is V(10) seconds.
    type: int
  dispatcher_heartbeat_period:
    description:
      - The delay (in nanoseconds) for an agent to send a heartbeat to the dispatcher.
      - Docker default value is 5 seconds, which corresponds to a value of V(5000000000).
    type: int
  node_cert_expiry:
    description:
      - Automatic expiry for nodes certificates, given in nanoseconds.
      - Docker default value is 90 days, which corresponds to a value of V(7776000000000000).
    type: int
  name:
    description:
      - The name of the swarm.
    type: str
  labels:
    description:
      - User-defined key/value metadata.
      - Label operations in this module apply to the docker swarm cluster. Use M(community.docker.docker_node) module to add/modify/remove
        swarm node labels.
      - Requires API version >= 1.32.
    type: dict
  signing_ca_cert:
    description:
      - The desired signing CA certificate for all swarm node TLS leaf certificates, in PEM format.
      - This must not be a path to a certificate, but the contents of the certificate.
      - Requires API version >= 1.30.
    type: str
  signing_ca_key:
    description:
      - The desired signing CA key for all swarm node TLS leaf certificates, in PEM format.
      - This must not be a path to a key, but the contents of the key.
      - Requires API version >= 1.30.
    type: str
  ca_force_rotate:
    description:
      - An integer whose purpose is to force swarm to generate a new signing CA certificate and key, if none have been specified.
      - Docker default value is V(0).
      - Requires API version >= 1.30.
    type: int
  autolock_managers:
    description:
      - If set, generate a key and use it to lock data stored on the managers.
      - Docker default value is V(false).
      - M(community.docker.docker_swarm_info) can be used to retrieve the unlock key.
    type: bool
  rotate_worker_token:
    description: Rotate the worker join token.
    type: bool
    default: false
  rotate_manager_token:
    description: Rotate the manager join token.
    type: bool
    default: false
  data_path_addr:
    description:
      - Address or interface to use for data path traffic.
      - This can either be an address in the form V(192.168.1.1), or an interface, like V(eth0).
      - Only used when swarm is initialised or joined. Because of this it is not considered for idempotency checking.
      - Requires API version >= 1.30.
    type: str
    version_added: 2.5.0
  data_path_port:
    description:
      - Port to use for data path traffic.
      - This needs to be a port number like V(9789).
      - Only used when swarm is initialised. Because of this it is not considered for idempotency checking.
      - Requires API version >= 1.40.
    type: int
    version_added: 3.1.0

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 2.0.0"
  - Docker API >= 1.25
author:
  - Thierry Bouvet (@tbouvet)
  - Piotr Wojciechowski (@WojciechowskiPiotr)
"""

EXAMPLES = r"""
---
- name: Init a new swarm with default parameters
  community.docker.docker_swarm:
    state: present

- name: Update swarm configuration
  community.docker.docker_swarm:
    state: present
    election_tick: 5

- name: Add nodes
  community.docker.docker_swarm:
    state: join
    advertise_addr: 192.168.1.2
    join_token: SWMTKN-1--xxxxx
    remote_addrs: ['192.168.1.1:2377']

- name: Leave swarm for a node
  community.docker.docker_swarm:
    state: absent

- name: Remove a swarm manager
  community.docker.docker_swarm:
    state: absent
    force: true

- name: Remove node from swarm
  community.docker.docker_swarm:
    state: remove
    node_id: mynode

- name: Init a new swarm with different data path interface
  community.docker.docker_swarm:
    state: present
    advertise_addr: eth0
    data_path_addr: ens10

- name: Init a new swarm with a different data path port
  community.docker.docker_swarm:
    state: present
    data_path_port: 9789
"""

RETURN = r"""
swarm_facts:
  description: Information about swarm.
  returned: success
  type: dict
  contains:
    JoinTokens:
      description: Tokens to connect to the Swarm.
      returned: success
      type: dict
      contains:
        Worker:
          description:
            - Token to join the cluster as a new *worker* node.
            - B(Note:) if this value has been specified as O(join_token), the value here will not be the token, but C(VALUE_SPECIFIED_IN_NO_LOG_PARAMETER).
              If you pass O(join_token), make sure your playbook/role does not depend on this return value!
          returned: success
          type: str
          example: SWMTKN-1--xxxxx
        Manager:
          description:
            - Token to join the cluster as a new *manager* node.
            - B(Note:) if this value has been specified as O(join_token), the value here will not be the token, but C(VALUE_SPECIFIED_IN_NO_LOG_PARAMETER).
              If you pass O(join_token), make sure your playbook/role does not depend on this return value!
          returned: success
          type: str
          example: SWMTKN-1--xxxxx
    UnlockKey:
      description: The swarm unlock-key if O(autolock_managers=true).
      returned: on success if O(autolock_managers=true) and swarm is initialised, or if O(autolock_managers) has changed.
      type: str
      example: SWMKEY-1-xxx

actions:
  description: Provides the actions done on the swarm.
  returned: when action failed.
  type: list
  elements: str
  example: ['This cluster is already a swarm cluster']
"""

import json
import traceback
import typing as t

try:
    from docker.errors import APIError, DockerException
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass

from ansible_collections.community.docker.plugins.module_utils._common import (
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._swarm import (
    AnsibleDockerSwarmClient,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DifferenceTracker,
    DockerBaseClass,
    sanitize_labels,
)


class TaskParameters(DockerBaseClass):
    def __init__(self) -> None:
        super().__init__()

        self.advertise_addr: str | None = None
        self.listen_addr: str | None = None
        self.remote_addrs: list[str] | None = None
        self.join_token: str | None = None
        self.data_path_addr: str | None = None
        self.data_path_port: int | None = None
        self.spec = None

        # Spec
        self.snapshot_interval: int | None = None
        self.task_history_retention_limit: int | None = None
        self.keep_old_snapshots: int | None = None
        self.log_entries_for_slow_followers: int | None = None
        self.heartbeat_tick: int | None = None
        self.election_tick: int | None = None
        self.dispatcher_heartbeat_period: int | None = None
        self.node_cert_expiry: int | None = None
        self.name: str | None = None
        self.labels: dict[str, t.Any] | None = None
        self.log_driver = None
        self.signing_ca_cert: str | None = None
        self.signing_ca_key: str | None = None
        self.ca_force_rotate: int | None = None
        self.autolock_managers: bool | None = None
        self.rotate_worker_token: bool | None = None
        self.rotate_manager_token: bool | None = None
        self.default_addr_pool: list[str] | None = None
        self.subnet_size: int | None = None

    @staticmethod
    def from_ansible_params(client: AnsibleDockerSwarmClient) -> TaskParameters:
        result = TaskParameters()
        for key, value in client.module.params.items():
            if key in result.__dict__:
                setattr(result, key, value)

        result.update_parameters(client)
        return result

    def update_from_swarm_info(self, swarm_info: dict[str, t.Any]) -> None:
        spec = swarm_info["Spec"]

        ca_config = spec.get("CAConfig") or {}
        if self.node_cert_expiry is None:
            self.node_cert_expiry = ca_config.get("NodeCertExpiry")
        if self.ca_force_rotate is None:
            self.ca_force_rotate = ca_config.get("ForceRotate")

        dispatcher = spec.get("Dispatcher") or {}
        if self.dispatcher_heartbeat_period is None:
            self.dispatcher_heartbeat_period = dispatcher.get("HeartbeatPeriod")

        raft = spec.get("Raft") or {}
        if self.snapshot_interval is None:
            self.snapshot_interval = raft.get("SnapshotInterval")
        if self.keep_old_snapshots is None:
            self.keep_old_snapshots = raft.get("KeepOldSnapshots")
        if self.heartbeat_tick is None:
            self.heartbeat_tick = raft.get("HeartbeatTick")
        if self.log_entries_for_slow_followers is None:
            self.log_entries_for_slow_followers = raft.get("LogEntriesForSlowFollowers")
        if self.election_tick is None:
            self.election_tick = raft.get("ElectionTick")

        orchestration = spec.get("Orchestration") or {}
        if self.task_history_retention_limit is None:
            self.task_history_retention_limit = orchestration.get(
                "TaskHistoryRetentionLimit"
            )

        encryption_config = spec.get("EncryptionConfig") or {}
        if self.autolock_managers is None:
            self.autolock_managers = encryption_config.get("AutoLockManagers")

        if self.name is None:
            self.name = spec["Name"]

        if self.labels is None:
            self.labels = spec.get("Labels") or {}

        if "LogDriver" in spec["TaskDefaults"]:
            self.log_driver = spec["TaskDefaults"]["LogDriver"]

    def update_parameters(self, client: AnsibleDockerSwarmClient) -> None:
        assign = {
            "snapshot_interval": "snapshot_interval",
            "task_history_retention_limit": "task_history_retention_limit",
            "keep_old_snapshots": "keep_old_snapshots",
            "log_entries_for_slow_followers": "log_entries_for_slow_followers",
            "heartbeat_tick": "heartbeat_tick",
            "election_tick": "election_tick",
            "dispatcher_heartbeat_period": "dispatcher_heartbeat_period",
            "node_cert_expiry": "node_cert_expiry",
            "name": "name",
            "labels": "labels",
            "signing_ca_cert": "signing_ca_cert",
            "signing_ca_key": "signing_ca_key",
            "ca_force_rotate": "ca_force_rotate",
            "autolock_managers": "autolock_managers",
            "log_driver": "log_driver",
        }
        params = {}
        for dest, source in assign.items():
            if not client.option_minimal_versions[source]["supported"]:
                continue
            value = getattr(self, source)
            if value is not None:
                params[dest] = value
        self.spec = client.create_swarm_spec(**params)

    def compare_to_active(
        self,
        other: TaskParameters,
        client: AnsibleDockerSwarmClient,
        differences: DifferenceTracker,
    ) -> DifferenceTracker:
        for k in self.__dict__:
            if k in (
                "advertise_addr",
                "listen_addr",
                "remote_addrs",
                "join_token",
                "rotate_worker_token",
                "rotate_manager_token",
                "spec",
                "default_addr_pool",
                "subnet_size",
                "data_path_addr",
                "data_path_port",
            ):
                continue
            if not client.option_minimal_versions[k]["supported"]:
                continue
            value = getattr(self, k)
            if value is None:
                continue
            other_value = getattr(other, k)
            if value != other_value:
                differences.add(k, parameter=value, active=other_value)
        if self.rotate_worker_token:
            differences.add("rotate_worker_token", parameter=True, active=False)
        if self.rotate_manager_token:
            differences.add("rotate_manager_token", parameter=True, active=False)
        return differences


class SwarmManager(DockerBaseClass):
    def __init__(
        self, client: AnsibleDockerSwarmClient, results: dict[str, t.Any]
    ) -> None:
        super().__init__()

        self.client = client
        self.results = results
        self.check_mode = self.client.check_mode
        self.swarm_info: dict[str, t.Any] = {}

        self.state: t.Literal["present", "join", "absent", "remove"] = (
            client.module.params["state"]
        )
        self.force: bool = client.module.params["force"]
        self.node_id: str | None = client.module.params["node_id"]

        self.differences = DifferenceTracker()
        self.parameters = TaskParameters.from_ansible_params(client)

        self.created = False

    def __call__(self) -> None:
        choice_map = {
            "present": self.init_swarm,
            "join": self.join,
            "absent": self.leave,
            "remove": self.remove,
        }

        choice_map[self.state]()

        if self.client.module._diff or self.parameters.debug:
            diff = {}
            diff["before"], diff["after"] = self.differences.get_before_after()
            self.results["diff"] = diff

    def inspect_swarm(self) -> None:
        try:
            data = self.client.inspect_swarm()
            json_str = json.dumps(data, ensure_ascii=False)
            self.swarm_info = json.loads(json_str)

            self.results["changed"] = False
            self.results["swarm_facts"] = self.swarm_info

            unlock_key = self.get_unlock_key()
            self.swarm_info.update(unlock_key)
        except APIError:
            pass

    def get_unlock_key(self) -> dict[str, t.Any]:
        default = {"UnlockKey": None}
        if not self.has_swarm_lock_changed():
            return default
        try:
            return self.client.get_unlock_key() or default
        except APIError:
            return default

    def has_swarm_lock_changed(self) -> bool:
        return bool(self.parameters.autolock_managers) and (
            self.created or self.differences.has_difference_for("autolock_managers")
        )

    def init_swarm(self) -> None:
        if not self.force and self.client.check_if_swarm_manager():
            self.__update_swarm()
            return

        if not self.check_mode:
            init_arguments: dict[str, t.Any] = {
                "advertise_addr": self.parameters.advertise_addr,
                "listen_addr": self.parameters.listen_addr,
                "force_new_cluster": self.force,
                "swarm_spec": self.parameters.spec,
            }
            if self.parameters.default_addr_pool is not None:
                init_arguments["default_addr_pool"] = self.parameters.default_addr_pool
            if self.parameters.subnet_size is not None:
                init_arguments["subnet_size"] = self.parameters.subnet_size
            if self.parameters.data_path_addr is not None:
                init_arguments["data_path_addr"] = self.parameters.data_path_addr
            if self.parameters.data_path_port is not None:
                init_arguments["data_path_port"] = self.parameters.data_path_port
            try:
                self.client.init_swarm(**init_arguments)
            except APIError as exc:
                self.client.fail(f"Can not create a new Swarm Cluster: {exc}")

        if not self.client.check_if_swarm_manager() and not self.check_mode:
            self.client.fail("Swarm not created or other error!")

        self.created = True
        self.inspect_swarm()
        self.results["actions"].append(
            f"New Swarm cluster created: {self.swarm_info.get('ID')}"
        )
        self.differences.add("state", parameter="present", active="absent")
        self.results["changed"] = True
        self.results["swarm_facts"] = {
            "JoinTokens": self.swarm_info.get("JoinTokens"),
            "UnlockKey": self.swarm_info.get("UnlockKey"),
        }

    def __update_swarm(self) -> None:
        try:
            self.inspect_swarm()
            version = self.swarm_info["Version"]["Index"]
            self.parameters.update_from_swarm_info(self.swarm_info)
            old_parameters = TaskParameters()
            old_parameters.update_from_swarm_info(self.swarm_info)
            self.parameters.compare_to_active(
                old_parameters, self.client, self.differences
            )
            if self.differences.empty:
                self.results["actions"].append("No modification")
                self.results["changed"] = False
                return
            update_parameters = TaskParameters.from_ansible_params(self.client)
            update_parameters.update_parameters(self.client)
            if not self.check_mode:
                self.client.update_swarm(
                    version=version,
                    swarm_spec=update_parameters.spec,
                    rotate_worker_token=self.parameters.rotate_worker_token,
                    rotate_manager_token=self.parameters.rotate_manager_token,
                )
        except APIError as exc:
            self.client.fail(f"Can not update a Swarm Cluster: {exc}")

        self.inspect_swarm()
        self.results["actions"].append("Swarm cluster updated")
        self.results["changed"] = True

    def join(self) -> None:
        if self.client.check_if_swarm_node():
            self.results["actions"].append("This node is already part of a swarm.")
            return
        if not self.check_mode:
            join_arguments = {
                "remote_addrs": self.parameters.remote_addrs,
                "join_token": self.parameters.join_token,
                "listen_addr": self.parameters.listen_addr,
                "advertise_addr": self.parameters.advertise_addr,
            }
            if self.parameters.data_path_addr is not None:
                join_arguments["data_path_addr"] = self.parameters.data_path_addr
            try:
                self.client.join_swarm(**join_arguments)
            except APIError as exc:
                self.client.fail(f"Can not join the Swarm Cluster: {exc}")
        self.results["actions"].append("New node is added to swarm cluster")
        self.differences.add("joined", parameter=True, active=False)
        self.results["changed"] = True

    def leave(self) -> None:
        if not self.client.check_if_swarm_node():
            self.results["actions"].append("This node is not part of a swarm.")
            return
        if not self.check_mode:
            try:
                self.client.leave_swarm(force=self.force)
            except APIError as exc:
                self.client.fail(f"This node can not leave the Swarm Cluster: {exc}")
        self.results["actions"].append("Node has left the swarm cluster")
        self.differences.add("joined", parameter="absent", active="present")
        self.results["changed"] = True

    def remove(self) -> None:
        if not self.client.check_if_swarm_manager():
            self.client.fail("This node is not a manager.")

        try:
            status_down = self.client.check_if_swarm_node_is_down(
                node_id=self.node_id, repeat_check=5
            )
        except APIError:
            return

        if not status_down:
            self.client.fail(
                "Can not remove the node. The status node is ready and not down."
            )

        if not self.check_mode:
            try:
                self.client.remove_node(node_id=self.node_id, force=self.force)
            except APIError as exc:
                self.client.fail(
                    f"Can not remove the node from the Swarm Cluster: {exc}"
                )
        self.results["actions"].append("Node is removed from swarm cluster.")
        self.differences.add("joined", parameter=False, active=True)
        self.results["changed"] = True


def _detect_remove_operation(client: AnsibleDockerSwarmClient) -> bool:
    return client.module.params["state"] == "remove"


def main() -> None:
    # TODO: missing option log_driver?
    argument_spec = {
        "advertise_addr": {"type": "str"},
        "data_path_addr": {"type": "str"},
        "data_path_port": {"type": "int"},
        "state": {
            "type": "str",
            "default": "present",
            "choices": ["present", "join", "absent", "remove"],
        },
        "force": {"type": "bool", "default": False},
        "listen_addr": {"type": "str", "default": "0.0.0.0:2377"},
        "remote_addrs": {"type": "list", "elements": "str"},
        "join_token": {"type": "str", "no_log": True},
        "snapshot_interval": {"type": "int"},
        "task_history_retention_limit": {"type": "int"},
        "keep_old_snapshots": {"type": "int"},
        "log_entries_for_slow_followers": {"type": "int"},
        "heartbeat_tick": {"type": "int"},
        "election_tick": {"type": "int"},
        "dispatcher_heartbeat_period": {"type": "int"},
        "node_cert_expiry": {"type": "int"},
        "name": {"type": "str"},
        "labels": {"type": "dict"},
        "signing_ca_cert": {"type": "str"},
        "signing_ca_key": {"type": "str", "no_log": True},
        "ca_force_rotate": {"type": "int"},
        "autolock_managers": {"type": "bool"},
        "node_id": {"type": "str"},
        "rotate_worker_token": {"type": "bool", "default": False},
        "rotate_manager_token": {"type": "bool", "default": False},
        "default_addr_pool": {"type": "list", "elements": "str"},
        "subnet_size": {"type": "int"},
    }

    required_if = [
        ("state", "join", ["remote_addrs", "join_token"]),
        ("state", "remove", ["node_id"]),
    ]

    option_minimal_versions = {
        "labels": {"docker_py_version": "2.6.0", "docker_api_version": "1.32"},
        "signing_ca_cert": {"docker_py_version": "2.6.0", "docker_api_version": "1.30"},
        "signing_ca_key": {"docker_py_version": "2.6.0", "docker_api_version": "1.30"},
        "ca_force_rotate": {"docker_py_version": "2.6.0", "docker_api_version": "1.30"},
        "autolock_managers": {"docker_py_version": "2.6.0"},
        "log_driver": {"docker_py_version": "2.6.0"},
        "remove_operation": {
            "docker_py_version": "2.4.0",
            "detect_usage": _detect_remove_operation,
            "usage_msg": "remove swarm nodes",
        },
        "default_addr_pool": {
            "docker_py_version": "4.0.0",
            "docker_api_version": "1.39",
        },
        "subnet_size": {"docker_py_version": "4.0.0", "docker_api_version": "1.39"},
        "data_path_addr": {"docker_py_version": "4.0.0", "docker_api_version": "1.30"},
        "data_path_port": {"docker_py_version": "6.0.0", "docker_api_version": "1.40"},
    }

    client = AnsibleDockerSwarmClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=required_if,
        min_docker_version="2.0.0",
        option_minimal_versions=option_minimal_versions,
    )
    sanitize_labels(client.module.params["labels"], "labels", client)

    try:
        results = {"changed": False, "result": "", "actions": []}

        SwarmManager(client, results)()
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when Docker SDK for Python tried to talk to the docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()
